@@ -127,7 +127,7 @@ then
     exit 1
 fi
 
-# Previously XC host was provided as a positional argument. 
+# Previously XC host was provided as a positional argument.
 if test -n "$legacy_xc_spec"
 then
     SBCL_XC_HOST="$legacy_xc_spec"
@@ -210,6 +210,19 @@ Options:
                   Use an existing CMU CL binary as a cross-compilation
                   host when you have weird things in your .cmucl-init
                   file.
+
+  --host-location=<string> Location of the source directory on compilation host
+
+      The string is passed to the command rsync to transfer the
+      necessary files between the target and host directories during
+      the make-target-*.sh steps of cross-compilation (cf. make.sh)
+
+      Examples:
+
+       user@host-machine:/home/user/sbcl
+                  Transfer the files to/from directory /home/user/sbcl
+                  on host-machine.
+
 EOF
   exit 1
 fi
@@ -300,7 +313,7 @@ case `uname` in
     SunOS)
         sbcl_os="sunos"
         ;;
-    CYGWIN* | WindowsNT | MINGW*)
+    CYGWIN* | WindowsNT | MINGW* | MSYS*)
         sbcl_os="win32"
         ;;
     HP-UX)
@@ -369,6 +382,7 @@ case `uname -m` in
     9000/800) guessed_sbcl_arch=hppa ;;
     mips*) guessed_sbcl_arch=mips ;;
     arm*) guessed_sbcl_arch=arm ;;
+    aarch64) guessed_sbcl_arch=arm ;;
     *)
         # If we're not building on a supported target architecture, we
         # we have no guess, but it's not an error yet, since maybe
@@ -417,6 +431,14 @@ then
             echo "No threads on this platform."
             ;;
     esac
+else
+    case $sbcl_arch in
+        x86|x86-64)
+            case $sbcl_os in
+                linux)
+                    WITH_FEATURES="$WITH_FEATURES :sb-thread"
+            esac
+    esac
 fi
 
 ltf=`pwd`/local-target-features.lisp-expr
@@ -424,8 +446,8 @@ echo //initializing $ltf
 echo ';;;; This is a machine-generated file.' > $ltf
 echo ';;;; Please do not edit it by hand.' >> $ltf
 echo ';;;; See make-config.sh.' >> $ltf
-echo "(lambda (features) (union (set-difference features (list$WITHOUT_FEATURES))" >> $ltf
-printf " (union (list$WITH_FEATURES) (list " >> $ltf
+echo "(lambda (features) (union (set-difference (union features (list$WITH_FEATURES)) (list$WITHOUT_FEATURES))" >> $ltf
+printf " (list " >> $ltf
 
 printf ":%s" "$sbcl_arch" >> $ltf
 
@@ -460,7 +482,7 @@ case "$sbcl_os" in
 		printf ' :largefile' >> $ltf
 		;;
             x86 | x86-64)
-		printf ' :sb-thread :sb-futex :largefile' >> $ltf
+		printf ' :sb-futex :largefile' >> $ltf
 		;;
             ppc)
 		printf ' :sb-futex' >> $ltf
@@ -725,7 +747,7 @@ export sbcl_os sbcl_arch
 sh tools-for-build/grovel-features.sh >> $ltf
 
 echo //finishing $ltf
-echo '))))' >> $ltf
+echo ')))' >> $ltf
 
 # FIXME: The version system should probably be redone along these lines:
 #
