@@ -35,8 +35,7 @@
 
 (declaim (inline singleton-p))
 (defun singleton-p (list)
-  (and (consp list)
-       (null (rest list))))
+  (and (listp list) (null (rest list)) list))
 
 #+sb-xc-host
 (progn
