@@ -101,6 +101,8 @@
 
 ;;; a list of toplevel things set by GENESIS
 (defvar *!reversed-cold-toplevels*)
+(!defvar *!reversed-cold-setf-macros* nil) ; just SETF macros
+(!defvar *!reversed-cold-defuns* nil)      ; just DEFUNs
 
 ;;; a SIMPLE-VECTOR set by GENESIS
 (defvar *!load-time-values*)
@@ -131,6 +133,11 @@
                       *standard-output* *error-output*
                       *trace-output* *error-output*))
 
+  ;; Assert that FBOUNDP doesn't choke when its answer is NIL.
+  ;; It was fine if T because in that case the legality of the arg is certain.
+  ;; And be extra paranoid - ensure that it really gets called.
+  (locally (declare (notinline fboundp)) (fboundp '(setf !zzzzzz)))
+
   ;; Putting data in a synchronized hashtable (*PACKAGE-NAMES*)
   ;; requires that the main thread be properly initialized.
   (show-and-call thread-init-or-reinit)
@@ -174,7 +181,6 @@
   (show-and-call sb!kernel::!late-type-cold-init2)
   (show-and-call !alien-type-cold-init)
   (show-and-call !target-type-cold-init)
-  (show-and-call !vm-type-cold-init)
   ;; FIXME: It would be tidy to make sure that that these cold init
   ;; functions are called in the same relative order as the toplevel
   ;; forms of the corresponding source files.
@@ -188,6 +194,12 @@
   ;; to the subclasses of STRUCTURE-OBJECT.
   (show-and-call sb!kernel::!set-up-structure-object-class)
 
+  (dolist (x (nreverse *!reversed-cold-setf-macros*))
+    (apply #'!quietly-defsetf x))
+  (dolist (x (nreverse *!reversed-cold-defuns*))
+    (destructuring-bind (name &optional docstring . inline-expansion) x
+      (!%quietly-defun name docstring inline-expansion)))
+
   ;; KLUDGE: Why are fixups mixed up with toplevel forms? Couldn't
   ;; fixups be done separately? Wouldn't that be clearer and better?
   ;; -- WHN 19991204
@@ -205,16 +217,9 @@
                       (let ((hexstr (hexstr r-c-tl-length)))
                         (/show0 "(hexstr calculated..)")
                         (/primitive-print hexstr))))
-  (let (#!+sb-show (index-in-cold-toplevels 0)
-        (really-note-if-setf-fun-and-macro #'sb!c::note-if-setf-fun-and-macro)
-        (really-assign-setf-macro #'sb!impl::assign-setf-macro))
+  (let (#!+sb-show (index-in-cold-toplevels 0))
     #!+sb-show (declare (type fixnum index-in-cold-toplevels))
 
-    (setf (symbol-function 'sb!c::note-if-setf-fun-and-macro)
-          (lambda (name) (declare (ignore name)) (values))
-          (symbol-function 'sb!impl::assign-setf-macro)
-          #'sb!impl::!quietly-assign-setf-macro)
-
     (dolist (toplevel-thing (prog1
                                 (nreverse *!reversed-cold-toplevels*)
                               ;; (Now that we've NREVERSEd it, it's
@@ -241,16 +246,11 @@
                                 (third toplevel-thing))
                   (get-lisp-obj-address
                    (svref *!load-time-values* (fourth toplevel-thing)))))
-           (defun
-            (destructuring-bind (name &optional docstring &rest inline-expansion)
-                (cdr toplevel-thing)
-              (!%quietly-defun name (fdefinition name) docstring inline-expansion)))
+           (defstruct
+            (apply 'sb!kernel::%defstruct (cdr toplevel-thing)))
            (t
             (!cold-lose "bogus fixup code in *!REVERSED-COLD-TOPLEVELS*"))))
-        (t (!cold-lose "bogus function in *!REVERSED-COLD-TOPLEVELS*"))))
-    (setf (symbol-function 'sb!c::note-if-setf-fun-and-macro)
-          really-note-if-setf-fun-and-macro
-          (symbol-function 'sb!impl::assign-setf-macro) really-assign-setf-macro))
+        (t (!cold-lose "bogus operation in *!REVERSED-COLD-TOPLEVELS*")))))
   (/show0 "done with loop over cold toplevel forms and fixups")
 
   ;; Set sane values again, so that the user sees sane values instead
@@ -302,8 +302,7 @@
   ;; the ANSI-specified initial value of *PACKAGE*
   (setf *package* (find-package "COMMON-LISP-USER"))
 
-  (/show0 "done initializing, setting *COLD-INIT-COMPLETE-P*")
-  (setf *cold-init-complete-p* t)
+  (!enable-infinite-error-protector)
 
   ; hppa heap is segmented, lisp and c uses a stub to call eachother
   #!+hpux (%primitive sb!vm::setup-return-from-lisp-stub)
