@@ -484,6 +484,20 @@ thread, NIL otherwise."
 ;;; Since arg-count checking happens before any of the stack locations
 ;;; and registers are overwritten all the arguments, including the
 ;;; extra ones, can be precisely recovered.
+#!+precise-arg-count-error
+(defun arg-count-error-frame-nth-arg (n frame)
+  (let* ((escaped (sb!di::compiled-frame-escaped frame))
+         (pointer (sb!di::frame-pointer frame))
+         (arg-count (sb!di::sub-access-debug-var-slot
+                     pointer sb!c:arg-count-sc escaped)))
+    (if (and (>= n 0)
+             (< n arg-count))
+        (sb!di::sub-access-debug-var-slot
+         pointer
+         (sb!c:standard-arg-location-sc n)
+         escaped)
+        (error "Index ~a out of bounds for ~a supplied argument~:p." n arg-count))))
+
 #!+precise-arg-count-error
 (defun arg-count-error-frame-args (frame)
   (let* ((escaped (sb!di::compiled-frame-escaped frame))
@@ -491,10 +505,10 @@ thread, NIL otherwise."
          (arg-count (sb!di::sub-access-debug-var-slot
                      pointer sb!c:arg-count-sc escaped)))
     (loop for i below arg-count
-          collect  (sb!di::sub-access-debug-var-slot
-                    pointer
-                    (sb!c:standard-arg-location-sc i)
-                    escaped))))
+          collect (sb!di::sub-access-debug-var-slot
+                   pointer
+                   (sb!c:standard-arg-location-sc i)
+                   escaped))))
 
 (defun frame-args-as-list (frame)
   #!+precise-arg-count-error
@@ -1233,6 +1247,13 @@ forms that explicitly control this kind of evaluation.")
 ;;; These commands are functions, not really commands, so that users
 ;;; can get their hands on the values returned.
 
+(defun var-valid-in-frame-p (var location &optional (frame *current-frame*))
+  ;; arg count errors are checked before anything is set up but they
+  ;; are reporeted in *elsewhere*, which is after start-pc saved in the
+  ;; debug function, defeating the checks.
+  (and (not (sb!di::tl-invalid-arg-count-error-p frame))
+       (eq (sb!di:debug-var-validity var location) :valid)))
+
 (eval-when (:execute :compile-toplevel)
 
 (sb!xc:defmacro define-var-operation (ref-or-set &optional value-var)
@@ -1246,8 +1267,7 @@ forms that explicitly control this kind of evaluation.")
           (location (sb!di:frame-code-location *current-frame*))
           ;; Let's only deal with valid variables.
           (vars (remove-if-not (lambda (v)
-                                 (eq (sb!di:debug-var-validity v location)
-                                     :valid))
+                                 (var-valid-in-frame-p v location))
                                temp)))
      (declare (list vars))
      (cond ((null vars)
@@ -1384,6 +1404,10 @@ forms that explicitly control this kind of evaluation.")
   "Return the N'th argument's value if possible. Argument zero is the first
    argument in a frame's default printed representation. Count keyword/value
    pairs as separate arguments."
+  #!+precise-arg-count-error
+  (when (sb!di::tl-invalid-arg-count-error-p *current-frame*)
+    (return-from arg
+      (arg-count-error-frame-nth-arg n *current-frame*)))
   (multiple-value-bind (var lambda-var-p)
       (nth-arg n (handler-case (sb!di:debug-fun-lambda-list
                                 (sb!di:frame-debug-fun *current-frame*))
@@ -1610,7 +1634,7 @@ forms that explicitly control this kind of evaluation.")
                       d-fun
                       (if prefix (string prefix) "")))
             (setf any-p t)
-            (when (eq (sb!di:debug-var-validity v location) :valid)
+            (when (var-valid-in-frame-p v location)
               (setf any-valid-p t)
               (case (sb!di::debug-var-info v)
                 (:more-context
