@@ -292,7 +292,8 @@
 ;;; in the functional position, including macros and lambdas.
 (defmacro collect (collections &body body)
   (let ((macros ())
-        (binds ()))
+        (binds ())
+        (ignores ()))
     (dolist (spec collections)
       (unless (proper-list-of-length-p spec 1 3)
         (error "malformed collection specifier: ~S" spec))
@@ -307,6 +308,7 @@
           (let ((n-tail (gensym (concatenate 'string
                                              (symbol-name name)
                                              "-N-TAIL-"))))
+            (push n-tail ignores)
             (if default
               (push `(,n-tail (last ,n-value)) binds)
               (push n-tail binds))
@@ -316,7 +318,13 @@
           (push `(,name (&rest args)
                    (collect-normal-expander ',n-value ',kind args))
                 macros))))
-    `(macrolet ,macros (let* ,(nreverse binds) ,@body))))
+    `(macrolet ,macros
+       (let* ,(nreverse binds)
+         ;; Even if the user reads each collection result,
+         ;; reader conditionals might statically eliminate all writes.
+         ;; Since we don't know, all the -n-tail variable are ignorable.
+         ,@(if ignores `((declare (ignorable ,@ignores))))
+         ,@body))))
 
 ;;;; some old-fashioned functions. (They're not just for old-fashioned
 ;;;; code, they're also used as optimized forms of the corresponding
@@ -476,6 +484,77 @@
              (setq ,val (pop ,tail))
              (progn ,@body)))))
 
+;;; (binding* ({(names initial-value [flag])}*) body)
+;;; FLAG may be NIL or :EXIT-IF-NULL
+;;;
+;;; This form unites LET*, MULTIPLE-VALUE-BIND and AWHEN.
+;;; Any name in a list of names may be NIL to ignore the respective value.
+;;; If NAMES itself is nil, the initial-value form is evaluated only for effect.
+;;;
+;;; Clauses with no flag and one binding are equivalent to LET.
+;;;
+;;; Caution: don't use declarations of the form (<non-builtin-type-id> <var>)
+;;; before the INFO database is set up in building the cross-compiler,
+;;; or you will probably lose.
+;;; Of course, since some other host Lisps don't seem to think that's
+;;; acceptable syntax anyway, you're pretty much prevented from writing it.
+;;;
+(defmacro binding* ((&rest clauses) &body body)
+  (unless clauses ; wrap in LET to preserve non-toplevelness
+    (return-from binding* `(let () ,@body)))
+  (multiple-value-bind (body decls) (parse-body body :doc-string-allowed nil)
+    ;; Generate an abstract representation that combines LET* clauses.
+    (let (repr)
+      (dolist (clause clauses)
+        (destructuring-bind (symbols value-form &optional flag) clause
+          (declare (type (member :exit-if-null nil) flag))
+          (let* ((ignore nil)
+                 (symbols
+                  (cond ((not (listp symbols)) (list symbols))
+                        ((not symbols) (setq ignore (list (gensym))))
+                        (t (mapcar
+                            (lambda (x) (or x (car (push (gensym) ignore))))
+                            symbols))))
+                 (flags (logior (if (cdr symbols) 1 0) (if flag 2 0)))
+                 (last (car repr)))
+            ;; EVENP => this clause does not entail multiple-value-bind
+            (cond ((and (evenp flags) (eql (car last) 0))
+                   (setf (first last) flags)
+                   (push (car symbols) (second last))
+                   (push value-form (third last))
+                   (setf (fourth last) (nconc ignore (fourth last))))
+                  (t
+                   (push (list flags symbols (list value-form) ignore)
+                         repr))))))
+      ;; Starting with the innermost binding clause, snarf out the
+      ;; applicable declarations. (Clauses are currently reversed)
+      (dolist (abstract-clause repr)
+        (when decls
+          (multiple-value-bind (binding-decls remaining-decls)
+              (extract-var-decls decls (second abstract-clause))
+            (setf (cddddr abstract-clause) binding-decls)
+            (setf decls remaining-decls))))
+      ;; Generate sexprs from inside out.
+      (loop with listp = t ; BODY is already a list
+            for (flags symbols values ignore . binding-decls) in repr
+            ;; Maybe test the last bound symbol in the clause for LET*
+            ;; or 1st symbol for mv-bind. Either way, the first of SYMBOLS.
+            for inner = (if (logtest flags 2) ; :EXIT-IF-NULL was specified.
+                            (prog1 `(when ,(car symbols)
+                                      ,@(if listp body (list body)))
+                              (setq listp nil))
+                            body)
+         do (setq body
+                  `(,.(if (evenp flags)
+                          `(let* ,(nreverse (mapcar #'list symbols values)))
+                          `(multiple-value-bind ,symbols ,(car values)))
+                    ,@(when binding-decls (list binding-decls))
+                    ,@(when ignore `((declare (ignorable ,@ignore))))
+                    ,@decls ; anything leftover
+                    ,@(if listp inner (list inner)))
+                  listp nil
+                  decls nil))
+      body)))
 
 ;;;; hash cache utility
 
@@ -564,24 +643,24 @@
 ;; so a 1-arg/1-result cache line needn't cons at all except once
 ;; (and maybe not even that if we make the cache into pairs of cells).
 ;; But this way is easier to understand, for now anyway.
+(eval-when (#-sb-xc :compile-toplevel :load-toplevel :execute)
+  (defun hash-cache-line-allocator (n)
+    (aref #.(coerce (loop for i from 2 to 6
+                          collect (symbolicate "ALLOC-HASH-CACHE-LINE/"
+                                               (char "23456" (- i 2))))
+                    'vector)
+          (- n 2))))
 (macrolet ((def (n)
              (let* ((ftype `(sfunction ,(make-list n :initial-element t) t))
-                    (fn (symbolicate "ALLOC-HASH-CACHE-LINE/"
-                                     (write-to-string n)))
-                    (args (loop for i from 1 to n
-                                collect (make-symbol (write-to-string i)))))
+                    (fn (hash-cache-line-allocator n))
+                    (args (make-gensym-list n)))
                `(progn
                   (declaim (ftype ,ftype ,fn))
                   (defun ,fn ,args
                     (declare (optimize (safety 0)))
                     ,(if (<= n 3)
                          `(list* ,@args)
-                         ;; FIXME: (VECTOR ,@args) should emit exactly the
-                         ;; same code as this, except it is worse.
-                         `(let ((a (make-array ,n)))
-                            ,@(loop for i from 0 for arg in args
-                                    collect `(setf (svref a ,i) ,arg))
-                            a)))))))
+                         `(vector ,@args)))))))
   (def 2)
   (def 3)
   (def 4)
@@ -657,9 +736,7 @@
                        (setq ,hashval (ash ,hashval ,(- hash-bits)))))))
                (multiple-value-bind ,result-temps (funcall ,thunk)
                  (let ((,entry
-                        (,(let ((*package* (symbol-package 'alloc-hash-cache)))
-                            (symbolicate "ALLOC-HASH-CACHE-LINE/"
-                                         (write-to-string (+ nargs values))))
+                        (,(hash-cache-line-allocator (+ nargs values))
                          ,@arg-vars ,@result-temps))
                        (,cache
                         (truly-the ,cache-type
@@ -711,10 +788,9 @@
                                         memoizer-supplied-p)
                               &allow-other-keys)
                         args &body body-decls-doc)
-  (let ((arg-names (mapcar #'car args)))
-    ;; What I wouldn't give to be able to use BINDING*, right?
-    (multiple-value-bind (forms decls doc) (parse-body body-decls-doc)
-      `(progn
+  (binding* (((forms decls doc) (parse-body body-decls-doc))
+             (arg-names (mapcar #'car args)))
+    `(progn
         (!define-hash-cache ,name ,args ,@options)
         (defun ,name ,arg-names
           ,@decls
@@ -727,7 +803,7 @@
                          (lambda () ,@body) ,@',arg-names)))
              ,@(if memoizer-supplied-p
                    forms
-                   `((,memoizer ,@forms)))))))))
+                   `((,memoizer ,@forms))))))))
 
 ;;; FIXME: maybe not the best place
 ;;;
@@ -1332,8 +1408,14 @@
 ;; Given DECLS as returned by from parse-body, and SYMBOLS to be bound
 ;; (with LET, MULTIPLE-VALUE-BIND, etc) return two sets of declarations:
 ;; those which pertain to the variables and those which don't.
+;; The first returned value is NIL or a single expression headed by DECLARE.
+;; The second is a list of expressions resembling the input DECLS.
 (defun extract-var-decls (decls symbols)
+  (unless symbols ; Don't bother filtering DECLS, just return them.
+    (return-from extract-var-decls (values nil decls)))
   (labels ((applies-to-variables (decl)
+             ;; If DECL is a variable-affecting declaration, then return
+             ;; the subset of SYMBOLS to which DECL applies.
              (let ((id (car decl)))
                (remove-if (lambda (x) (not (memq x symbols)))
                           (cond ((eq id 'type)
@@ -1345,17 +1427,18 @@
                                      (info :type :kind id))
                                  (cdr decl))))))
            (partition (spec)
-             (let ((variables (applies-to-variables spec)))
-               (cond ((not variables)
-                      (values nil spec))
-                     ((eq (car spec) 'type)
-                      (let ((more (set-difference (cddr spec) variables)))
-                        (values `(type ,(cadr spec) ,@variables)
-                                (if more `(type ,(cadr spec) ,@more)))))
-                     (t
-                      (let ((more (set-difference (cdr spec) variables)))
-                        (values `(,(car spec) ,@variables)
-                                (if more `(,(car spec) ,@more)))))))))
+             ;; If SPEC is a declaration affecting some variables in SYMBOLS
+             ;; and some not, split it into two mutually exclusive declarations.
+             (acond ((applies-to-variables spec)
+                     (multiple-value-bind (decl-head all-symbols)
+                         (if (eq (car spec) 'type)
+                             (values `(type ,(cadr spec)) (cddr spec))
+                             (values `(,(car spec)) (cdr spec)))
+                       (let ((more (set-difference all-symbols it)))
+                         (values `(,@decl-head ,@it)
+                                 (and more `(,@decl-head ,@more))))))
+                    (t
+                     (values nil spec)))))
     ;; This loop is less inefficient than theoretically possible,
     ;; reconstructing the tree even if no need,
     ;; but it's just a macroexpander, so... fine.
@@ -1372,92 +1455,6 @@
                      decls)))
         (values (awhen (binding-decls) `(declare ,@it))
                 (mapcan (lambda (x) (if x (list `(declare ,@x)))) filtered))))))
-
-;;; (binding* ({(names initial-value [flag])}*) body)
-;;; FLAG may be NIL or :EXIT-IF-NULL
-;;;
-;;; This form unites LET*, MULTIPLE-VALUE-BIND and AWHEN.
-;;; Any name in a list of names may be NIL to ignore the respective value.
-;;; If NAMES itself is nil, the initial-value form is evaluated only for effect.
-;;;
-;;; Clauses with no flags and one binding per clause are equivalent to LET*.
-;;; We reduce to LET* when possible so that the body can contain declarations
-;;; without having to split out declarations which affect variables and insert
-;;; them into the appropriate places. This qualifies as an extreme KLUDGE,
-;;; but has desirable behavior of allowing declarations in the innermost form.
-;;;
-;;; Caution: don't use declarations of the form (<non-builtin-type-id> <var>)
-;;; before the INFO database is set up in building the cross-compiler,
-;;; or you will probably lose.
-;;; Of course, since some other host Lisps don't seem to think that's
-;;; acceptable syntax anyway, you're pretty much prevented from writing it.
-;;;
-(defmacro binding* ((&rest bindings) &body body)
-  (multiple-value-bind (forms decls) (parse-body body :doc-string-allowed nil)
-    (labels
-      ((recurse (bindings decls &aux ignores)
-         (cond
-           ((some (lambda (x)
-                    (destructuring-bind (names value-form &optional flag) x
-                      (declare (ignore value-form))
-                      (or flag (not (symbolp names)))))
-                  bindings)
-            (destructuring-bind (names value-form &optional flag) (car bindings)
-              (etypecase names
-                ;; () for names is esoteric. Does anyone really need that?
-                (null   (setq names (list (gensym)) ignores names))
-                (symbol (setq names (list names)))
-                (list
-                 (setq names (mapcar (lambda (name)
-                                       (or name (car (push (gensym) ignores))))
-                                     names))))
-              (multiple-value-bind (binding-decls rest-decls)
-                  ;; If no more bindings, and no (WHEN ...) before the FORMS,
-                  ;; then don't bother parsing decls.
-                  (if (or (cdr bindings) flag)
-                      (extract-var-decls decls
-                                         (filter-names names (cdr bindings)))
-                      (values nil decls))
-                (let ((continue (acond ((cdr bindings) (recurse it rest-decls))
-                                       (t (append decls forms)))))
-                  `((multiple-value-bind ,names ,value-form
-                      ,@(decl-expr binding-decls ignores)
-                      ,@(ecase flag
-                          ((nil) continue)
-                          ((:exit-if-null)
-                           `((when ,(first names) ,@continue))))))))))
-           (t
-            ;; This case is not strictly necessary now that declarations that
-            ;; affect variables are correctly inserted into the M-V-BIND,
-            ;; but it makes the expansion more legible/concise when applicable.
-            `((let* ,(mapcar (lambda (binding)
-                               (if (car binding)
-                                   binding
-                                   (let ((var (gensym)))
-                                     (push var ignores)
-                                     (cons var (cdr binding)))))
-                             bindings)
-                ,@(decl-expr nil ignores)
-                ,@body)))))
-       (filter-names (names more-bindings)
-         ;; Return the subset of SYMBOLs that does not intersect any
-         ;; symbol in MORE-BINDINGS. This makes declarations apply only
-         ;; to the final occurrence of a repeated name, as is the custom.
-         (remove-if (lambda (x) (subsequently-bound-p x more-bindings)) names))
-       (subsequently-bound-p (name more-bindings)
-         (member-if (lambda (binding)
-                      (let ((names (car binding)))
-                        (if (listp names) (memq name names) (eq name names))))
-                    more-bindings))
-       (decl-expr (binding-decls ignores)
-         (nconc (if binding-decls (list binding-decls))
-         ;; IGNORABLE, not IGNORE, just in case :EXIT-IF-NULL reads a gensym
-               (if ignores `((declare (ignorable ,@ignores)))))))
-    ;; Zero bindings have to be special-cased. RECURSE returns a list of forms
-    ;; because we musn't wrap BODY in a PROGN if it contains declarations,
-    ;; so we unwrap once here, but if the body was returned as the base case
-    ;; of recursion then (CAR (RECURSE)) would be wrong.
-    (if bindings (car (recurse bindings decls)) `(locally ,@body)))))
 
 ;;; Delayed evaluation
 (defmacro delay (form)
@@ -1576,13 +1573,12 @@ to :INTERPRET, an interpreter will be used.")
     ;; Sort by descending seek count to rank by likely relative importance
     (dolist (symbol (sort (copy-list *cache-vector-symbols*) #'>
                           :key (lambda (x) (aref (cache-stats x) 0))))
-      ;; Sadly we can't use BINDING* within this file
-      (multiple-value-bind (stats short-name) (cache-stats symbol)
-        (let* ((seek (aref stats 0))
-               (miss (aref stats 1))
-               (hit (- seek miss))
-               (evict (aref stats 2))
-               (cache (symbol-value symbol)))
+      (binding* (((stats short-name) (cache-stats symbol))
+                 (seek (aref stats 0))
+                 (miss (aref stats 1))
+                 (hit (- seek miss))
+                 (evict (aref stats 2))
+                 (cache (symbol-value symbol)))
           (format t "~9d ~9d (~5,1f%) ~8d (~5,1f%) ~4d ~6,1f% ~A~%"
                   seek hit
                   (if (plusp seek) (* 100 (/ hit seek)))
@@ -1592,7 +1588,7 @@ to :INTERPRET, an interpreter will be used.")
                   (if (plusp (length cache))
                       (* 100 (/ (count-if-not #'fixnump cache)
                                 (length cache))))
-                  short-name)))))))
+                  short-name))))))
 
 (in-package "SB!KERNEL")
 
@@ -1641,7 +1637,7 @@ to :INTERPRET, an interpreter will be used.")
 
 (defun stream-error-position-info (stream &optional position)
   (when (and (not position) (form-tracking-stream-p stream))
-    (let ((line/col (sb!c::line/col-from-charpos stream)))
+    (let ((line/col (line/col-from-charpos stream)))
       (return-from stream-error-position-info
         `((:line ,(car line/col))
           (:column ,(cdr line/col))
@@ -1715,4 +1711,5 @@ lines and columns."
         `(let ((,var (make-string-output-stream)))
            ,@decls
            ,@forms
-           (get-output-stream-string ,var)))))
+           (truly-the (simple-array character (*))
+                      (get-output-stream-string ,var))))))
