@@ -105,17 +105,19 @@
                                        (subseq optional 0 (1+ last-not-rest))))
                                 rest))))
 
-(defun parse-args-types (lambda-list-like-thing)
-  (multiple-value-bind
-        (required optional restp rest keyp keys allowp auxp aux
-                  morep more-context more-count llk-p)
-      (parse-lambda-list-like-thing lambda-list-like-thing :silent t)
-    (declare (ignore aux morep more-context more-count))
-    (when auxp
-      (error "&AUX in a FUNCTION or VALUES type: ~S." lambda-list-like-thing))
+(defun parse-args-types (lambda-listy-thing context)
+  (multiple-value-bind (llks required optional rest keys)
+      (parse-lambda-list
+       lambda-listy-thing
+       :context context
+       :accept (ecase context
+                 (:values-type (lambda-list-keyword-mask '(&optional &rest)))
+                 (:function-type (lambda-list-keyword-mask
+                                  '(&optional &rest &key &allow-other-keys))))
+       :silent t)
     (let ((required (mapcar #'single-value-specifier-type required))
           (optional (mapcar #'single-value-specifier-type optional))
-          (rest (when restp (single-value-specifier-type rest)))
+          (rest (when rest (single-value-specifier-type (car rest))))
           (keywords
            (collect ((key-info))
              (dolist (key keys)
@@ -124,15 +126,16 @@
                (let ((kwd (first key)))
                  (when (find kwd (key-info) :key #'key-info-name)
                    (error "~@<repeated keyword ~S in lambda list: ~2I~_~S~:>"
-                          kwd lambda-list-like-thing))
+                          kwd lambda-listy-thing))
                  (key-info
                   (make-key-info
                    :name kwd
                    :type (single-value-specifier-type (second key))))))
              (key-info))))
       (multiple-value-bind (required optional rest)
-          (canonicalize-args-type-args required optional rest keyp)
-        (values required optional rest keyp keywords allowp llk-p)))))
+          (canonicalize-args-type-args required optional rest
+                                       (ll-kwds-keyp llks))
+        (values llks required optional rest keywords)))))
 
 (defstruct (values-type
             (:include args-type
@@ -862,9 +865,11 @@
               (when (and (atom spec)
                          (member spec '(and or not member eql satisfies values)))
                 (error "The symbol ~S is not valid as a type specifier." spec))
-              (let ((fun (info :type :translator (if (consp spec) (car spec) spec))))
-                (cond (fun
-                       (funcall fun (if (atom spec) (list spec) spec)))
+              (let ((fun-or-ctype
+                     (info :type :translator (if (consp spec) (car spec) spec))))
+                (cond ((functionp fun-or-ctype)
+                       (funcall fun-or-ctype (if (atom spec) (list spec) spec)))
+                      (fun-or-ctype)
                       ((or (and (consp spec) (symbolp (car spec))
                                 (not (info :type :builtin (car spec))))
                            (and (symbolp spec) (not (info :type :builtin spec))))
@@ -921,9 +926,9 @@ Returns two values: the expansion, and a boolean that is true when
 expansion happened."
   (declare (type type-specifier type-specifier))
   (declare (ignore env))
-  (multiple-value-bind (expander lspec)
-      (let ((spec type-specifier))
-        (cond ((and (symbolp spec) (info :type :builtin spec))
+  (let* ((spec type-specifier)
+         (atom (if (listp spec) (car spec) spec))
+         (expander (and (symbolp atom) (info :type :expander atom))))
                ;; We do not expand builtins even though it'd be
                ;; possible to do so sometimes (e.g. STRING) for two
                ;; reasons:
@@ -931,20 +936,8 @@ expansion happened."
                ;; a) From a user's point of view, CL types are opaque.
                ;;
                ;; b) so (EQUAL (TYPEXPAND 'STRING) (TYPEXPAND-ALL 'STRING))
-               (values nil nil))
-              ((symbolp spec)
-               (values (info :type :expander spec) spec))
-              ((and (consp spec) (symbolp (car spec)) (info :type :builtin (car spec)))
-               ;; see above
-               (values nil nil))
-              ((and (consp spec) (symbolp (car spec)))
-               (values (info :type :expander (car spec)) spec))
-              (t nil)))
-    (if expander
-        (values (funcall expander (if (symbolp lspec)
-                                      (list lspec)
-                                      lspec))
-                t)
+    (if (and expander (not (info :type :builtin atom)))
+        (values (funcall expander (if (symbolp spec) (list spec) spec)) t)
         (values type-specifier nil))))
 
 (defun typexpand (type-specifier &optional env)
