@@ -44,12 +44,12 @@
 (defmacro program-destructuring-bind (lambda-list arg-list &body body)
   (let ((arg-list-name (gensym "ARG-LIST-")))
     (multiple-value-bind (body local-decls)
-        (sb!kernel:parse-defmacro lambda-list arg-list-name body nil
-                                  'program-destructuring-bind
-                                  :anonymousp t
-                                  :doc-string-allowed nil
-                                  :wrap-block nil
-                                  :error-fun 'arg-count-program-error)
+        (parse-defmacro lambda-list arg-list-name body nil
+                        'program-destructuring-bind
+                        :anonymousp t
+                        :doc-string-allowed nil
+                        :wrap-block nil
+                        :error-fun 'arg-count-program-error)
       `(let ((,arg-list-name ,arg-list))
          ,@local-decls
          ,body))))
@@ -104,7 +104,7 @@
                    nil nil nil nil nil
                    (sb!c::lexenv-handled-conditions old-lexenv)
                    (sb!c::lexenv-disabled-package-locks old-lexenv)
-                   (sb!c::lexenv-policy old-lexenv)
+                   (sb!c::lexenv-policy old-lexenv) ; = (OR %POLICY *POLICY*)
                    (sb!c::lexenv-user-data old-lexenv))))
       (dolist (declaration declarations)
         (unless (consp declaration)
@@ -112,30 +112,32 @@
                     declaration (cons 'declare declarations)))
         (case (car declaration)
           ((optimize)
+           (setf (sb!c::lexenv-%policy lexenv)
+                 (copy-structure (sb!c::lexenv-%policy lexenv)))
            (dolist (element (cdr declaration))
              (multiple-value-bind (quality value)
-                 (if (not (consp element))
+                 (if (not (consp element)) ; FIXME: OAOOM w/'proclaim'
                      (values element 3)
                      (program-destructuring-bind (quality value)
                          element
                        (values quality value)))
-               (if (sb!c::policy-quality-name-p quality)
-                   (push (cons quality value)
-                         (sb!c::lexenv-%policy lexenv))
-                   (warn "ignoring unknown optimization quality ~
-                                      ~S in ~S" quality
-                                      (cons 'declare declarations))))))
-          (sb!ext:muffle-conditions
+               (sb!int:acond
+                ((sb!c::policy-quality-name-p quality)
+                 (sb!c::alter-policy (sb!c::lexenv-%policy lexenv)
+                                     sb!int:it value))
+                (t (warn "ignoring unknown optimization quality ~S in ~S"
+                         quality (cons 'declare declarations)))))))
+          (muffle-conditions
            (setf (sb!c::lexenv-handled-conditions lexenv)
                  (sb!c::process-muffle-conditions-decl
                   declaration
                   (sb!c::lexenv-handled-conditions lexenv))))
-          (sb!ext:unmuffle-conditions
+          (unmuffle-conditions
            (setf (sb!c::lexenv-handled-conditions lexenv)
                  (sb!c::process-unmuffle-conditions-decl
                   declaration
                   (sb!c::lexenv-handled-conditions lexenv))))
-          ((sb!ext:disable-package-locks sb!ext:enable-package-locks)
+          ((disable-package-locks sb!ext:enable-package-locks)
            (setf (sb!c::lexenv-disabled-package-locks lexenv)
                  (sb!c::process-package-lock-decl
                   declaration
@@ -282,11 +284,15 @@
 ;;;
 ;;; Used only for implementing calls to interpreted functions.
 (defun parse-arguments (arguments lambda-list)
-  (multiple-value-bind (required optional rest-p rest keyword-p
-                        keyword allow-other-keys-p aux-p aux)
+  (multiple-value-bind (llks required optional rest keyword aux)
+      ;; FIXME: shouldn't this just pass ":silent t" ?
       (handler-bind ((style-warning #'muffle-warning))
         (sb!int:parse-lambda-list lambda-list))
     (let* ((original-arguments arguments)
+           (rest-p (not (null rest)))
+           (rest (car rest))
+           (keyword-p (sb!int:ll-kwds-keyp llks))
+           (allow-other-keys-p (sb!int:ll-kwds-allowp llks))
            (arguments-present (length arguments))
            (required-length (length required))
            (optional-length (length optional))
@@ -345,7 +351,7 @@
                 (push (cons (supplied-p-parameter keyword-spec)
                             (list 'quote (not (eq supplied *not-present*))))
                       let*-like-bindings))))))
-      (when aux-p
+      (when aux
         (do ()
             ((null aux))
           (let ((this-aux (pop aux)))
@@ -357,9 +363,10 @@
 ;;; Evaluate LET*-like (sequential) bindings.
 ;;;
 ;;; Given an alist of BINDINGS, evaluate the value form of the first
-;;; binding in ENV, bind the variable to the value in ENV, and then
-;;; evaluate the next binding form. Once all binding forms have been
-;;; handled, END-ACTION is funcalled.
+;;; binding in ENV, generate an augmented environment with a binding
+;;; of the variable to the value in ENV, and then evaluate the next
+;;; binding form. Once all binding forms have been handled, END-ACTION
+;;; is funcalled with the final environment.
 ;;;
 ;;; SPECIALS is a list of variables that have a bound special declaration.
 ;;; These variables (and those that have been declaimed as special) are
@@ -373,20 +380,21 @@
                (%eval exp env))))
     (if bindings
         (let* ((binding-name (car (car bindings)))
-               (binding-value (cdr (car bindings))))
+               (binding-value (cdr (car bindings)))
+               (new-env (make-env :parent env)))
           (if (specialp binding-name specials)
               (progv
                   (list binding-name)
                   (list (maybe-eval binding-value))
                 ;; Mark the variable as special in this environment
-                (push-var binding-name *special* env)
-                (eval-next-let*-binding (cdr bindings)
-                                        specials env end-action))
+                (push-var binding-name *special* new-env)
+                (eval-next-let*-binding
+                 (cdr bindings) specials new-env end-action))
               (progn
-                (push-var binding-name (maybe-eval binding-value) env)
-                (eval-next-let*-binding (cdr bindings)
-                                        specials env end-action))))
-        (funcall end-action))))
+                (push-var binding-name (maybe-eval binding-value) new-env)
+                (eval-next-let*-binding
+                 (cdr bindings) specials new-env end-action))))
+        (funcall end-action env))))
 
 ;;; Create a new environment based on OLD-ENV by adding the variable
 ;;; bindings in BINDINGS to it, and call FUNCTION with the new environment
@@ -461,7 +469,7 @@
            ;; Then deal with optionals / keywords / etc.
            (eval-next-let*-binding
             let*-like-binding var-specials env
-            #'(lambda ()
+            #'(lambda (env)
                 ;; And now that we have evaluated all the
                 ;; initialization forms for the bindings, add the free
                 ;; special declarations to the environment. To see why
@@ -500,7 +508,7 @@
           (t
            (let ((type (sb!c::info :variable :type symbol)))
              (when type
-               (let ((type-specifier (sb!kernel:type-specifier type)))
+               (let ((type-specifier (type-specifier type)))
                  (unless (typep value type-specifier)
                    (error 'type-error
                           :datum value
@@ -646,7 +654,7 @@
                              (binding-value binding)))
                    bindings)
            var-specials env
-           #'(lambda ()
+           #'(lambda (env)
                ;; Now that we're done evaluating the bindings, add the
                ;; free special declarations. See also
                ;; CALL-WITH-NEW-ENV-FULL-PARSING.
@@ -1038,7 +1046,7 @@
        ((throw)                (eval-throw (cdr exp) env))
        ((unwind-protect)       (eval-unwind-protect (cdr exp) env))
        ;; SBCL-specific:
-       ((sb!ext:truly-the)     (eval-the (cdr exp) env))
+       ((truly-the)            (eval-the (cdr exp) env))
        ;; Not a special form, but a macro whose expansion wouldn't be
        ;; handled correctly by the evaluator.
        ((sb!sys:with-pinned-objects) (eval-with-pinned-objects (cdr exp) env))
@@ -1194,6 +1202,6 @@
             (%eval form env))
         (compiler-environment-too-complex-error (condition)
           (declare (ignore condition))
-          (sb!int:style-warn 'sb!kernel:lexical-environment-too-complex
+          (sb!int:style-warn 'lexical-environment-too-complex
                              :form form :lexenv lexenv)
           (sb!int:simple-eval-in-lexenv form lexenv))))))
