@@ -180,20 +180,17 @@ invoked. In that case it will store into PLACE and start over."
     (multiple-value-bind (body local-decs doc)
         (parse-defmacro lambda-list whole body name 'define-compiler-macro
                         :environment environment)
-      (let ((def `(lambda (,whole ,environment)
-                    ,@(sb!c:macro-policy-decls t)
+      (let ((def `(named-lambda
+                      ,(sb!c::debug-name 'compiler-macro-function name)
+                      (,whole ,environment)
+                    ,@(sb!c:macro-policy-decls)
                     ,@local-decs
-                    ,body))
-            (debug-name (sb!c::debug-name 'compiler-macro-function name)))
+                    ,body)))
         `(progn
           (eval-when (:compile-toplevel)
            (sb!c::%compiler-defmacro :compiler-macro-function ',name t))
           (eval-when (:compile-toplevel :load-toplevel :execute)
-           (sb!c::%define-compiler-macro ',name
-                                         #',def
-                                         ',lambda-list
-                                         ,doc
-                                         ',debug-name)))))))
+           (sb!c::%define-compiler-macro ',name ,def ',lambda-list ,doc)))))))
 
 ;;; FIXME: This will look remarkably similar to those who have already
 ;;; seen the code for %DEFMACRO in src/code/defmacro.lisp.  Various
@@ -201,18 +198,16 @@ invoked. In that case it will store into PLACE and start over."
 (macrolet
     ((def (times set-p)
          `(eval-when (,@times)
-           (defun sb!c::%define-compiler-macro
-               (name definition lambda-list doc debug-name)
+           (defun sb!c::%define-compiler-macro (name definition lambda-list doc)
              ,@(unless set-p
-                 '((declare (ignore lambda-list doc debug-name))))
+                 '((declare (ignore lambda-list doc))))
              (sb!c::warn-if-compiler-macro-dependency-problem name)
              ;; FIXME: warn about incompatible lambda list with
              ;; respect to parent function?
              (setf (sb!xc:compiler-macro-function name) definition)
              ,(when set-p
-                    `(setf (%fun-doc definition) doc
-                           (%fun-lambda-list definition) lambda-list
-                           (%fun-name definition) debug-name))
+                `(setf (%fun-doc definition) doc
+                       (%fun-lambda-list definition) lambda-list))
              name))))
   (progn
     (def (:load-toplevel :execute) #-sb-xc-host t #+sb-xc-host nil)
@@ -523,7 +518,8 @@ invoked. In that case it will store into PLACE and start over."
   "DECLAIM Declaration*
   Do a declaration or declarations for the global environment."
   `(eval-when (:compile-toplevel :load-toplevel :execute)
-     ,@(mapcar (lambda (spec) `(sb!xc:proclaim ',spec))
+     ,@(mapcar (lambda (spec)
+                 `(sb!c::%proclaim ',spec (sb!c:source-location)))
                specs)))
 
 ;; Avoid unknown return values in emitted code for PRINT-UNREADABLE-OBJECT
