@@ -145,10 +145,7 @@
                   (defmacro-error (format nil "required argument after ~A"
                                           restp)
                       context name))
-                (when (process-sublist var "REQUIRED-" `(car ,path))
-                  ;; Note &ENVIRONMENT from DEFSETF sublist
-                  (aver (eq context 'defsetf))
-                  (setf env-arg-used t))
+                (process-sublist var "REQUIRED-" `(car ,path))
                 (setq path `(cdr ,path)
                       minimum (1+ minimum)
                       maximum (1+ maximum)))
@@ -203,9 +200,7 @@
                (&environment
                 (cond (env-illegal
                        (error "&ENVIRONMENT is not valid with ~S." context))
-                      ;; DEFSETF explicitly allows &ENVIRONMENT, and we get
-                      ;; it here in a sublist.
-                      ((and sublist (neq context 'defsetf))
+                      (sublist
                        (error "&ENVIRONMENT is only valid at top level of ~
                              lambda-list."))
                       (env-arg-used
@@ -342,15 +337,15 @@
 
 ;;; We save space in macro definitions by calling this function.
 (defun arg-count-error (context name args lambda-list minimum maximum)
-  (let (#-sb-xc-host
-        (sb!debug:*stack-top-hint* (or sb!debug:*stack-top-hint* 'arg-count-error)))
-    (error 'arg-count-error
+  ;; Tail-call ERROR, contrary to usual behavior.
+  #-sb-xc-host (declare (optimize sb!c::allow-non-returning-tail-call))
+  (error 'arg-count-error
            :kind context
            :name name
            :args args
            :lambda-list lambda-list
            :minimum minimum
-           :maximum maximum)))
+           :maximum maximum))
 
 (defun push-sublist-binding (variable path object name context error-fun)
   (check-defmacro-arg variable)
@@ -378,22 +373,24 @@
 
 (defun push-optional-binding (value-var init-form suppliedp-name
                               &key is-supplied-p path name context error-fun)
-  (unless suppliedp-name
-    (setq suppliedp-name (gensym "SUPPLIEDP-")))
-  (push-let-binding suppliedp-name is-supplied-p :system t)
-  (cond ((consp value-var)
-         (let ((whole-thing (gensym "OPTIONAL-SUBLIST-")))
-           (push-sublist-binding whole-thing
-                                 `(if ,suppliedp-name ,path ,init-form)
-                                 value-var name context error-fun)
-           (parse-defmacro-lambda-list value-var whole-thing name
-                                       context
-                                       :error-fun error-fun
-                                       :sublist t)))
-        ((symbolp value-var)
-         (push-let-binding value-var path :when suppliedp-name :else init-form))
-        (t
-         (error "illegal optional variable name: ~S" value-var))))
+  (let ((sym (gensym "SUPPLIEDP-")))
+    (push-let-binding sym is-supplied-p :system t)
+    (cond ((consp value-var)
+           (let ((whole-thing (gensym "OPTIONAL-SUBLIST-")))
+             (push-sublist-binding whole-thing
+                                   `(if ,sym ,path ,init-form)
+                                   value-var name context error-fun)
+             (parse-defmacro-lambda-list value-var whole-thing name
+                                         context
+                                         :error-fun error-fun
+                                         :sublist t)))
+          ((symbolp value-var)
+           (push-let-binding value-var path :when sym :else init-form))
+          (t
+           (error "Illegal optional variable name: ~S" value-var)))
+    ;; Shouldn't be bound during the initform evaluation
+    (when suppliedp-name
+      (push-let-binding suppliedp-name sym))))
 
 (defun defmacro-error (problem context name)
   (error "illegal or ill-formed ~A argument in ~A~@[ ~S~]"
