@@ -103,21 +103,13 @@
   (declare (optimize (speed 3) (safety 0)))
   (cond ((%instancep x) (%instance-layout x))
         ((funcallable-instance-p x) (%funcallable-instance-layout x))
-        ((null x)
-         ;; Note: was #.((CLASS-LAYOUT (SB!XC:FIND-CLASS 'NULL))).
-         ;; I (WHN 19990209) replaced this with an expression evaluated at
-         ;; run time in order to make it easier to build the cross-compiler.
-         ;;
-         ;; KLUDGE: Since there's a DEFTRANSFORM for FIND-CLASSOID on
-         ;; constant names which creates non-cold-loadable code, we
-         ;; can't just use (CLASSOID-LAYOUT (FIND-CLASSOID 'NULL))
-         ;; here. The original (WHN 19991004) solution was to locally
-         ;; notinline FIND-CLASSOID. However, the full call to
-         ;; FIND-CLASSOID caused suboptimal register allocation in PCL
-         ;; dfuns. So instead we now use a special variable which is
-         ;; initialized during cold init. -- JES, 2006-07-04
-         **null-classoid-layout**)
-        (t (svref **built-in-class-codes** (widetag-of x)))))
+        ;; Compiler can dump literal layouts, which handily sidesteps
+        ;; the question of when cold-init runs L-T-V forms.
+        ((null x) #.(find-layout 'null))
+        (t
+         ;; Note that WIDETAG-OF is slightly suboptimal here and could be
+         ;; improved - we've already ruled out some of the lowtags.
+         (svref (load-time-value **built-in-class-codes** t) (widetag-of x)))))
 
 #!-sb-fluid (declaim (inline classoid-of))
 (defun classoid-of (object)
@@ -143,10 +135,7 @@
 ;;; type checking, rather than trying to come up with the one that the
 ;;; user might find most informative.
 (declaim (ftype (function (t) ctype) ctype-of))
-(defun-cached (ctype-of
-               :hash-function #'sxhash
-               :hash-bits 9
-               :init-wrapper !cold-init-forms)
+(defun-cached (ctype-of :hash-function #'sxhash :hash-bits 9)
               ((x eq))
   (typecase x
     (function
@@ -159,7 +148,7 @@
      (ctype-of-number x))
     (array
      (let ((etype (specifier-type (array-element-type x))))
-       (make-array-type :dimensions (array-dimensions x)
+       (make-array-type (array-dimensions x)
                         :complexp (not (typep x 'simple-array))
                         :element-type etype
                         :specialized-element-type etype)))
