@@ -361,12 +361,10 @@ from the name currently assigned to CHARACTER. Otherwise, returns NIL.
 This property has been officially obsoleted by the Unicode standard, and
 is only included for backwards compatibility."
   (let* ((char-code (char-code character))
-         (h-code (cdr (binary-search char-code
-                                   (car **unicode-1-character-name-database**)
-                                   :key #'car))))
+         (h-code (double-vector-binary-search char-code
+                                              **unicode-1-char-name-database**)))
     (when h-code
-      ;; Remove UNICODE1_ prefix
-      (subseq (huffman-decode h-code **unicode-character-name-huffman-tree**) 9))))
+      (huffman-decode h-code **unicode-character-name-huffman-tree**))))
 
 (defun age (character)
   #!+sb-doc
@@ -516,7 +514,8 @@ disappears when accents are placed on top of it. and NIL otherwise"
                      (+ 4 (misc-index char)))))
     (values (clear-flag 7 value) (logbitp 7 value))))
 
-(defun char-decomposition (char length)
+(defun char-decomposition (char length callback)
+  (declare (function callback))
   ;; Caller should have gotten length from char-decomposition-info
   (let* ((cp (char-code char))
          (cp-high (ash cp -8))
@@ -524,85 +523,77 @@ disappears when accents are placed on top of it. and NIL otherwise"
          (high-page (aref **character-high-pages** cp-high))
          (index (unless (logbitp 15 high-page) ;; Hangul syllable
                   (aref **character-low-pages**
-                        (+ 1 (* 2 (+ (ldb (byte 8 0) cp) (ash high-page 8)))))))
-         (entry (when index (loop for i from 0 below length
-                               collecting (aref decompositions (+ i index)))))
-         (result (make-string length)))
-    (if (= length 1)
-        (string (code-char (car entry)))
-        (progn
-          (if (<= #xac00 cp #xd7a3)
-              ;; see Unicode 6.2, section 3-12
-              (let* ((sbase #xac00)
-                     (lbase #x1100)
-                     (vbase #x1161)
-                     (tbase #x11a7)
-                     (lcount 19)
-                     (vcount 21)
-                     (tcount 28)
-                     (ncount (* vcount tcount))
-                     (scount (* lcount ncount))
-                     (sindex (- cp sbase))
-                     (lindex (floor sindex ncount))
-                     (vindex (floor (mod sindex ncount) tcount))
-                     (tindex (mod sindex tcount)))
-                (declare (ignore scount))
-                (setf (char result 0) (code-char (+ lbase lindex)))
-                (setf (char result 1) (code-char (+ vbase vindex)))
-                (if (> tindex 0)
-                  (setf (char result 2) (code-char (+ tbase tindex)))
-                  (setf result (subseq result 0 2)))) ; Remove trailing #\Nul
-              (loop for i from 0 for code in entry
-                 do (setf (char result i) (code-char code))))
-          result))))
+                        (+ 1 (* 2 (+ (ldb (byte 8 0) cp) (ash high-page 8))))))))
+    (cond ((= length 1)
+           (funcall callback (code-char (aref decompositions index))))
+          ((<= #xac00 cp #xd7a3)
+           ;; see Unicode 6.2, section 3-12
+           (let* ((sbase #xac00)
+                  (lbase #x1100)
+                  (vbase #x1161)
+                  (tbase #x11a7)
+                  (vcount 21)
+                  (tcount 28)
+                  (ncount (* vcount tcount))
+                  (sindex (- cp sbase))
+                  (lindex (floor sindex ncount))
+                  (vindex (floor (mod sindex ncount) tcount))
+                  (tindex (mod sindex tcount)))
+             (funcall callback (code-char (+ lbase lindex)))
+             (funcall callback (code-char (+ vbase vindex)))
+             (when (> tindex 0)
+               (funcall callback  (code-char (+ tbase tindex))))))
 
-(defun decompose-char (char)
-  (let ((info (char-decomposition-info char)))
-    (if (= info 0)
-        (string char)
-        (char-decomposition char info))))
+          (t
+           (loop for i below length
+                 do
+                 (funcall callback (code-char (aref decompositions (+ index i)))))))))
+
+(defun decompose-char (char compatibility callback)
+  (declare (function callback))
+  (multiple-value-bind (info compat) (char-decomposition-info char)
+    (if (and (plusp info)
+             (or compatibility
+                 (not compat)))
+        (if compatibility
+            (dx-flet ((callback (char)
+                        (decompose-char char t callback)))
+              (char-decomposition char info #'callback))
+            (char-decomposition char info callback))
+        (funcall callback char))))
 
 (defun decompose-string (string &optional (kind :canonical))
-  (declare (type (member :canonical :compatibility) kind))
-  (flet ((canonical (char)
-           (multiple-value-bind (len compat) (char-decomposition-info char)
-             (and (/= len 0) (not compat))))
-         (compat (char)
-           (/= 0 (char-decomposition-info char))))
-    (let (result
-          (fun (ecase kind
-                 (:canonical #'canonical)
-                 (:compatibility #'compat))))
-      (do* ((start 0 (1+ end))
-            (end (position-if fun string :start start)
-                 (position-if fun string :start start)))
-           ((null end) (push (subseq string start end) result))
-        (unless (= start end)
-          (push (subseq string start end) result))
-        ;; FIXME: this recursive call to DECOMPOSE-STRING is necessary
-        ;; for correctness given our direct encoding of the
-        ;; decomposition data in UnicodeData.txt.  It would, however,
-        ;; be straightforward enough to perform the recursion in table
-        ;; construction, and then have this simply revert to a single
-        ;; lookup.  (Wait for tests to be hooked in, then implement).
-        (push (decompose-string (decompose-char (char string end)) kind)
-              result))
-      (apply 'concatenate 'string (nreverse result)))))
-
-(defun sort-combiners (string)
-  (let (result (start 0) first-cc first-non-cc)
-    (tagbody
-     again
-       (setf first-cc (position 0 string :key #'combining-class :test #'/= :start start))
-       (when first-cc
-         (setf first-non-cc (position 0 string :key #'combining-class :test #'= :start first-cc)))
-       (push (subseq string start first-cc) result)
-       (when first-cc
-         (push (stable-sort (subseq string first-cc first-non-cc) #'< :key #'combining-class) result))
-       (when first-non-cc
-         (setf start first-non-cc first-cc nil first-non-cc nil)
-         (go again)))
-    (apply 'concatenate 'string (nreverse result))))
+  (let ((compatibility (ecase kind
+                         (:compatibility t)
+                         (:canonical nil))))
+    (let (chars
+          (length 0)
+          previous-char
+          (previous-combining-class 0))
+      (dx-flet ((callback (char)
+                  (let ((combining-class (combining-class char)))
+                    (incf length)
+                    (cond ((< 0 combining-class previous-combining-class)
+                           ;; Ensure it's sorted
+                           (loop for cons on chars
+                                 for next-char = (cadr cons)
+                                 when (or (not next-char)
+                                          (<= 0 (combining-class next-char) combining-class))
+                                 do (setf (cdr cons)
+                                          (cons char (cdr cons)))
+                                    (return)))
+                          (t
+                           (push char chars)
+                           (setf previous-char char
+                                 previous-combining-class combining-class))))))
+        (loop for char across string
+              do
+              (decompose-char char compatibility #'callback))
+        (let ((result (make-string length)))
+          (loop for char in (nreverse chars)
+                for i from 0
+                do (setf (schar result i) char))
+          result)))))
 
 (defun composition-hangul-syllable-type (cp)
   (cond
@@ -689,57 +680,41 @@ disappears when accents are placed on top of it. and NIL otherwise"
     (push l so-fars)))
 
 (defun canonically-compose (string)
-  (labels ()
-    (let* ((result (list (list 0 (length string) string)))
-           (previous-starter-index (position 0 string :key #'combining-class))
-           (i (and previous-starter-index (1+ previous-starter-index))))
-      (when (or (not i) (= i (length string)))
-        (return-from canonically-compose string))
-      (tagbody
-       again
-         (when (and (>= (- i previous-starter-index) 2)
-                    ;; test for Blocked (Unicode 3.11 para. D115)
-                    ;;
-                    ;; (assumes here that string has sorted combiners,
-                    ;; so can look back just one step)
-                    (>= (combining-class (lref result (1- i)))
-                        (combining-class (lref result i))))
-           (when (= (combining-class (lref result i)) 0)
-             (setf previous-starter-index i))
-           (incf i)
-           (go next))
+  (let* ((result (list (list 0 (length string) string)))
+         (previous-starter-index (position 0 string :key #'combining-class))
+         (i (and previous-starter-index (1+ previous-starter-index))))
+    (when (or (not i) (= i (length string)))
+      (return-from canonically-compose string))
+    (tagbody
+     again
+       (when (and (>= (- i previous-starter-index) 2)
+                  ;; test for Blocked (Unicode 3.11 para. D115)
+                  ;;
+                  ;; (assumes here that string has sorted combiners,
+                  ;; so can look back just one step)
+                  (>= (combining-class (lref result (1- i)))
+                      (combining-class (lref result i))))
+         (when (= (combining-class (lref result i)) 0)
+           (setf previous-starter-index i))
+         (incf i)
+         (go next))
 
-         (let ((comp (primary-composition (lref result previous-starter-index)
-                                          (lref result i))))
-           (cond
-             (comp
-              (setf (lref result previous-starter-index) comp)
-              (setf result (ldelete result i)))
-             (t
-              (when (= (combining-class (lref result i)) 0)
-                (setf previous-starter-index i))
-              (incf i))))
-       next
-         (unless (= i (llength result))
-           (go again)))
-      (if (= i (length string))
-          string
-          (lstring result)))))
-
-(defun quick-normalization-check (string no-prop &optional maybe-prop)
-  (let ((prev-cc 0))
-    (loop for c across string
-       for cp = (char-code c)
-       for cc = (combining-class c)
-       do
-         (unless (<= #x100000 cp #x10FFFF)
-           (if (or
-                (and (> prev-cc cc) (/= cc 0))
-                (proplist-p c no-prop)
-                (proplist-p c maybe-prop))
-               (return-from quick-normalization-check nil)
-               (setf prev-cc cc))))
-    string))
+       (let ((comp (primary-composition (lref result previous-starter-index)
+                                        (lref result i))))
+         (cond
+           (comp
+            (setf (lref result previous-starter-index) comp)
+            (setf result (ldelete result i)))
+           (t
+            (when (= (combining-class (lref result i)) 0)
+              (setf previous-starter-index i))
+            (incf i))))
+     next
+       (unless (= i (llength result))
+         (go again)))
+    (if (= i (length string))
+        string
+        (lstring result))))
 
 (defun normalize-string (string &optional (form :nfd))
   #!+sb-doc
@@ -759,27 +734,19 @@ disappears when accents are placed on top of it. and NIL otherwise"
     ((array character (*))
      (ecase form
        ((:nfc)
-        (or
-         (quick-normalization-check string :nfc-qc :nfc-qc-maybe)
-         (canonically-compose (sort-combiners (decompose-string string)))))
+        (canonically-compose (decompose-string string)))
        ((:nfd)
-        (or
-         (quick-normalization-check string :nfd-qc)
-         (sort-combiners (decompose-string string))))
+        (decompose-string string))
        ((:nfkc)
-        (or
-         (quick-normalization-check string :nfkc-qc :nfkc-qc-maybe)
-         (canonically-compose (sort-combiners (decompose-string string :compatibility)))))
+        (canonically-compose (decompose-string string :compatibility)))
        ((:nfkd)
-        (or
-         (quick-normalization-check string :nfkd-qc)
-         (sort-combiners (decompose-string string :compatibility))))))
+        (decompose-string string :compatibility))))
     ((array nil (*)) string)))
 
 (defun normalized-p (string &optional (form :nfd))
   #!+sb-doc
   "Tests if STRING is normalized to FORM"
-  ;; This is fast because of the quick check
+  ;; FIXME: can be optimized
   (string= string (normalize-string string form)))
 
 
@@ -815,13 +782,13 @@ disappears when accents are placed on top of it. and NIL otherwise"
 
 (defun char-uppercase (char)
   (if (has-case-p char)
-      (let ((cp (car (lookup-char-case-info char))))
+      (let ((cp (car (char-case-info char))))
         (if (atom cp) (list (code-char cp)) (mapcar #'code-char cp)))
       (list char)))
 
 (defun char-lowercase (char)
   (if (has-case-p char)
-      (let ((cp (cdr (lookup-char-case-info char))))
+      (let ((cp (cdr (char-case-info char))))
         (if (atom cp) (list (code-char cp)) (mapcar #'code-char cp)))
       (list char)))
 
@@ -1532,105 +1499,117 @@ it defaults to 80 characters"
       (read stream)))
 
 (defun unpack-collation-key (key)
-  (loop for value across key collect
-       (list (ldb (byte 16 16) value)
-             (ldb (byte 11 5) value)
-             (ldb (byte 5 0) value))))
+  (declare (type (simple-array (unsigned-byte 32) (*)) key))
+  (loop for value across key
+        collect
+        (list (ldb (byte 16 16) value)
+              (ldb (byte 11 5) value)
+              (ldb (byte 5 0) value))))
 
-(defun variable-p (unpacked-key)
-  (<= 1 (car unpacked-key) **maximum-variable-primary-element**))
+(defun variable-p (x)
+  (<= 1 x **maximum-variable-primary-element**))
 
-(defun collation-key (char1 &optional char2 char3 &rest overflow)
-  ;; There are never more than three characters in a contraction, right?
-  (when overflow (return-from collation-key nil))
-  ;; hack around the inability of the cross-compiler to deal with characters
-  (when (not char2) (setf char2 (code-char 0)))
-  (when (not char3) (setf char3 (code-char 0)))
-  (let ((packed-key (gethash (pack-3-codepoints
-                              (char-code char1)
-                              (char-code char2)
-                              (char-code char3))
-                             **character-collations**)))
-    (if packed-key (unpack-collation-key packed-key)
-        (when (char= (code-char 0) char2 char3)
-          (let* ((cp (char-code char1))
-                 (base
-                  (if (proplist-p char1 :unified-ideograph)
-                     (if (or (<= #x4E00 cp #x9FFF)
-                             (<= #xF900 cp #xFAFF))
-                         #xFB40 #xFB80) #xFBC0))
-                 (a (+ base (ash cp -15)))
-                 (b (logior #.(ash 1 15) (logand cp #x7FFFF))))
-            (list (list a #x20 #x2) (list b 0 0)))))))
-
-(defun generic-apply (fn args)
-  (apply fn (coerce args 'list)))
+(defun collation-key (string start end)
+  (let (char1
+        (char2 (code-char 0))
+        (char3 (code-char 0)))
+    (case (- end start)
+      (1 (setf char1 (char string start)))
+      (2 (setf char1 (char string start)
+               char2 (char string (+ start 1))))
+      (3 (setf char1 (char string start)
+               char2 (char string (+ start 1))
+               char3 (char string (+ start 2))))
+      (t
+       ;; There are never more than three characters in a contraction, right?
+       (return-from collation-key nil)))
+    (let ((packed-key (gethash (pack-3-codepoints
+                                (char-code char1)
+                                (char-code char2)
+                                (char-code char3))
+                               **character-collations**)))
+      (if packed-key
+          (unpack-collation-key packed-key)
+          (when (char= (code-char 0) char2 char3)
+            (let* ((cp (char-code char1))
+                   (base
+                     (cond ((not (proplist-p char1 :unified-ideograph))
+                            #xFBC0)
+                           ((or (<= #x4E00 cp #x9FFF)
+                                (<= #xF900 cp #xFAFF))
+                            #xFB40)
+                           (t
+                            #xFB80)))
+                   (a (+ base (ash cp -15)))
+                   (b (logior #.(ash 1 15) (logand cp #x7FFFF))))
+              (list (list a #x20 #x2) (list b 0 0))))))))
 
 (defun sort-key (string)
-   (let* ((str (normalize-string string :nfd))
-          (i 0) (len (length str)) max-match new-i
-          sort-key)
-     (loop while (< i len) do
+  (let* ((str (normalize-string string :nfd))
+         (i 0) (len (length str)) max-match new-i
+         sort-key
+         after-variable)
+    (loop while (< i len)
+          do
           (loop for offset from 1 to 3
-               for index = (+ i offset)
-               while (<= index len)
-               when (generic-apply #'collation-key (subseq str i index))
-             do (setf max-match
-                      (generic-apply #'collation-key (subseq str i index))
-                      new-i index))
+                for index = (+ i offset)
+                while (<= index len)
+                do
+                (let ((key (collation-key str i index)))
+                  (when key
+                    (setf max-match key
+                          new-i index))))
           (loop for index from new-i below len
-             for char = (char str index)
-             until (eql 0 (combining-class char))
-             unless (and (>= (- index new-i) 1)
-                         ;; Combiners are sorted, we only have to look back
-                         ;; one step (see canonically-compose)
-                         (>= (combining-class (char str (1- index)))
-                             (combining-class char)))
-             do (rotatef (char str new-i) (char str index))
-               (if (generic-apply #'collation-key (subseq str i (1+ new-i)))
-                   (setf max-match
-                         (generic-apply #'collation-key (subseq str i (1+ new-i)))
-                         new-i (1+ new-i))
-                   (rotatef (char str new-i) (char str index))))
+                for char = (char str index)
+                for previous-combining-class = combining-class
+                for combining-class = (combining-class char)
+                until (eql combining-class 0)
+                unless (and (>= (- index new-i) 1)
+                            ;; Combiners are sorted, we only have to look back
+                            ;; one step (see canonically-compose)
+                            (>= (combining-class (char str (1- index)))
+                                combining-class))
+                do
+                (rotatef (char str new-i) (char str index))
+                (let ((key (collation-key str i (1+ new-i))))
+                  (if key
+                      (setf max-match key
+                            new-i (1+ new-i))
+                      (rotatef (char str new-i) (char str index)))))
           (loop for key in max-match do (push key sort-key))
           (setf i new-i))
-     (setf
-      sort-key
-      (let (after-variable)
-        (mapcar
-         #'(lambda (key)
-             (destructuring-bind (k1 k2 k3) key
-               (cond
-                 ((= k1 k2 k3 0) (list k1 k2 k3 0))
-                 ((and (/= k1 0) (variable-p key))
-                  (setf after-variable t)
-                  (list 0 0 0 k1))
-                 ((/= k1 0) (setf after-variable nil)
-                  (list k1 k2 k3 #xFFFF))
-                 ((and (= k1 0) (/= k3 0))
-                  (if after-variable
-                      (list 0 0 0 0)
-                      (list k1 k2 k3 #xFFFF))))))
-         (nreverse sort-key))))
-     (let (primary secondary tertiary quatenary)
-       (loop for (k1 k2 k3 k4) in sort-key do
-            (when (/= k1 0) (push k1 primary))
-            (when (/= k2 0) (push k2 secondary))
-            (when (/= k3 0) (push k3 tertiary))
-            (when (/= k4 0) (push k4 quatenary)))
-       (setf primary (nreverse primary)
-             secondary (nreverse secondary)
-             tertiary (nreverse tertiary)
-             quatenary (nreverse quatenary))
-       (concatenate 'vector primary (list 0) secondary (list 0)
-                    tertiary (list 0) quatenary))))
+    (macrolet ((push-non-zero (obj place)
+                 `(when (/= ,obj 0)
+                    (push ,obj ,place))))
+      (let (primary secondary tertiary quatenary)
+        (loop for (k1 k2 k3) in (nreverse sort-key)
+              do
+              (cond
+                ((= k1 k2 k3 0))
+                ((variable-p k1)
+                 (setf after-variable t)
+                 (push k1 quatenary))
+                ((/= k1 0)
+                 (setf after-variable nil)
+                 (push k1 primary)
+                 (push-non-zero k2 secondary)
+                 (push-non-zero k3 tertiary)
+                 (push #xFFFF quatenary))
+                ((/= k3 0)
+                 (unless after-variable
+                   (push-non-zero k2 secondary)
+                   (push k3 tertiary)
+                   (push #xFFFF quatenary)))))
+        (concatenate 'vector
+                     (nreverse primary) #(0) (nreverse secondary) #(0)
+                     (nreverse tertiary) #(0) (nreverse quatenary))))))
 
 (defun vector< (vector1 vector2)
   (loop for i across vector1
-     for j across vector2
-     do
-       (cond ((< i j) (return-from vector< t))
-             ((> i j) (return-from vector< nil))))
+        for j across vector2
+        do
+        (cond ((< i j) (return-from vector< t))
+              ((> i j) (return-from vector< nil))))
   ;; If there's no differences, shortest vector wins
   (< (length vector1) (length vector2)))
 
@@ -1665,9 +1644,7 @@ with variable-weight characters, as described in UTS #10"
          (s2 (subseq string2 start2 end2))
          (k1 (sort-key s1)) (k2 (sort-key s2)))
     (if (equalp k1 k2)
-        (flet ((to-codepoints (str)
-                 (map 'vector #'char-code (normalize-string str :nfd))))
-          (vector< (to-codepoints s1) (to-codepoints s2)))
+        (string< (normalize-string s1 :nfd) (normalize-string s2 :nfd))
         (vector< k1 k2))))
 
 (defun unicode<= (string1 string2 &key (start1 0) end1 (start2 0) end2)
