@@ -51,11 +51,14 @@
         #+sb-package-locks (sb-ext:unlock-package package)))))
 
 ;; Reinstate the pre-cold-init variable-defining macros.
-(macrolet ((def (wrapper real-name)
-             `(defmacro ,wrapper (&rest args) `(,',real-name ,@args))))
-  (def sb-impl::!defglobal defglobal)
-  (def sb-impl::!defparameter defparameter)
-  (def sb-impl::!defvar defvar))
+(let ((*package* (find-package "SB-INT")))
+  (flet ((def (real-name)
+           (let ((alias (sb-int:symbolicate "!" real-name)))
+             (export alias)
+             (setf (macro-function alias) (macro-function real-name)))))
+    (def 'sb-ext:defglobal)
+    (def 'defparameter)
+    (def 'defvar)))
 
 (export '(sb-int::def!method
           sb-int::!coerce-to-specialized
