@@ -413,6 +413,7 @@
         '(error "This should be eliminated as dead code."))))
 
 (defoptimizer (%local-alien-addr derive-type) ((info var))
+  (declare (ignore var))
   (if (constant-lvar-p info)
       (let* ((info (lvar-value info))
              (alien-type (local-alien-info-type info)))
@@ -444,6 +445,7 @@
 ;;;; %CAST
 
 (defoptimizer (%cast derive-type) ((alien type))
+  (declare (ignore alien))
   (or (when (constant-lvar-p type)
         (let ((alien-type (lvar-value type)))
           (when (alien-type-p alien-type)
@@ -609,6 +611,36 @@
     `(lambda (function ,@names)
        (alien-funcall (deref function) ,@names))))
 
+(eval-when (:compile-toplevel :load-toplevel :execute)
+  (defvar *saved-fp-and-pcs* nil)
+  ;; Can't use DECLAIM since always-bound is a non-standard declaration
+  (sb!xc:proclaim '(sb!ext:always-bound *saved-fp-and-pcs*)))
+
+#!+c-stack-is-control-stack
+(declaim (inline invoke-with-saved-fp-and-pc))
+#!+c-stack-is-control-stack
+(defun invoke-with-saved-fp-and-pc (fn)
+  (declare #-sb-xc-host (muffle-conditions compiler-note)
+           (optimize (speed 3)))
+  (let ((fp-and-pc (make-array 2 :element-type 'word)))
+    (declare (truly-dynamic-extent fp-and-pc))
+    (setf (aref fp-and-pc 0) (sb!kernel:get-lisp-obj-address
+                              (sb!kernel:%caller-frame))
+          (aref fp-and-pc 1) (sap-int (sb!kernel:%caller-pc)))
+    (let ((*saved-fp-and-pcs* (cons fp-and-pc *saved-fp-and-pcs*)))
+      (declare (truly-dynamic-extent *saved-fp-and-pcs*))
+      (funcall fn))))
+
+(defun find-saved-fp-and-pc (fp)
+  (dolist (x *saved-fp-and-pcs*)
+    (declare (type (simple-array word (2)) x))
+    (when (#!+stack-grows-downward-not-upward
+           sap>
+           #!-stack-grows-downward-not-upward
+           sap<
+           (int-sap (aref x 0)) fp)
+      (return (values (int-sap (aref x 0)) (int-sap (aref x 1)))))))
+
 (deftransform alien-funcall ((function &rest args) * * :node node :important t)
   (let ((type (lvar-type function)))
     (unless (alien-type-type-p type)
@@ -666,7 +698,7 @@
             ;; Remember this frame to make sure that we can get back
             ;; to it later regardless of how the foreign stack looks
             ;; like.
-            #!+:c-stack-is-control-stack
+            #!+c-stack-is-control-stack
             (when (policy node (= 3 alien-funcall-saves-fp-and-pc))
               (setf body `(invoke-with-saved-fp-and-pc (lambda () ,body))))
             (/noshow "returning from DEFTRANSFORM ALIEN-FUNCALL" (params) body)
@@ -688,6 +720,7 @@
 
 (defoptimizer (%alien-funcall ltn-annotate)
               ((function type &rest args) node ltn-policy)
+  (declare (ignore type ltn-policy))
   (setf (basic-combination-info node) :funny)
   (setf (node-tail-p node) nil)
   (annotate-ordinary-lvar function)
