@@ -67,10 +67,6 @@
     (notinline . :notinline)
     (maybe-inline . :maybe-inline)))
 
-;;; the lexical environment we are currently converting in
-(defvar *lexenv*)
-(declaim (type lexenv *lexenv*))
-
 ;;; *FREE-VARS* translates from the names of variables referenced
 ;;; globally to the LEAF structures for them. *FREE-FUNS* is like
 ;;; *FREE-VARS*, only it deals with function names.
@@ -105,10 +101,6 @@
 (defvar *current-component*)
 (defvar *delayed-ir1-transforms*)
 (defvar *eval-tlf-index*)
-(defvar *handled-conditions*)
-(defvar *disabled-package-locks*)
-(defvar *policy*)
-(defvar *macro-policy* nil)
 (defvar *dynamic-counts-tn*)
 (defvar *elsewhere*)
 (defvar *event-info*)
@@ -120,7 +112,6 @@
   (defvar *constant-segment*)
   (defvar *constant-table*)
   (defvar *constant-vector*))
-(defvar *lexenv*)
 (defvar *source-info*)
 (defvar *source-plist*)
 (defvar *source-namestring*)
@@ -165,12 +156,20 @@ the stack without triggering overflow protection.")
 
 ;;;; miscellaneous utilities
 
+;;; This is for "observers" who want to know if type names have been added.
+;;; Rather than registering listeners, they can detect changes by comparing
+;;; their stored nonce to the current nonce. Additionally the observers
+;;; can detect whether function definitions have occurred.
+(declaim (fixnum *type-cache-nonce*))
+(!defglobal *type-cache-nonce* 0)
+
 ;;; Delete any undefined warnings for NAME and KIND. This is for the
 ;;; benefit of the compiler, but it's sometimes called from stuff like
 ;;; type-defining code which isn't logically part of the compiler.
 (declaim (ftype (function ((or symbol cons) keyword) (values))
                 note-name-defined))
 (defun note-name-defined (name kind)
+  #-sb-xc-host (atomic-incf *type-cache-nonce*)
   ;; We do this BOUNDP check because this function can be called when
   ;; not in a compilation unit (as when loading top level forms).
   (when (boundp '*undefined-warnings*)
