@@ -35,7 +35,20 @@
 ;;; The instances of double expansion still remain, e.g. (fun (macro)),
 ;;; since PROCESS-TOPLEVEL-FORM only expands the macros at the first
 ;;; position.
-(defun fopcompilable-p (form &optional (expand t))
+
+(flet ((setq-fopcompilable-p (args)
+         (loop for (name value) on args by #'cddr
+               always (and (symbolp name)
+                           (member (info :variable :kind name)
+                                   '(:special :global))
+                           (fopcompilable-p value))))
+       (cold-svset-fopcompilable-p (args)
+         (destructuring-bind (thing index value) args
+           (and (symbolp thing)
+                (integerp index)
+                (eq (info :variable :kind thing) :global)
+                (typep value '(cons (eql function) (cons symbol null)))))))
+ (defun fopcompilable-p (form &optional (expand t))
   ;; We'd like to be able to handle
   ;;   -- simple funcalls, nested recursively, e.g.
   ;;      (SET '*PACKAGE* (FIND-PACKAGE "CL-USER"))
@@ -58,7 +71,25 @@
   ;; the cold loader too, providing definitions for A and B before
   ;; executing all other toplevel forms.
   #+sb-xc-host
-  (and expand (eq (car form) 'sb!impl::%defun))
+  (and expand
+       (or (and (self-evaluating-p form)
+                (constant-fopcompilable-p form))
+           (and (listp form)
+                (let ((function (car form)))
+                  ;; It is assumed that uses of recognized functions are
+                  ;; carefully controlled, and recursion on fopcompilable-p
+                  ;; would say "yes".
+                  (or (member function '(sb!impl::%defun
+                                         sb!impl::%defsetf
+                                         sb!kernel::%defstruct))
+                      (and (symbolp function) ; no ((lambda ...) ...)
+                           (not (null (get function :sb-cold-funcall-handler))))
+                      (and (eq function 'setf)
+                           (fopcompilable-p (%macroexpand form *lexenv*)))
+                      (and (eq function 'sb!kernel:%svset)
+                           (cold-svset-fopcompilable-p (cdr form)))
+                      (and (eq function 'setq)
+                           (setq-fopcompilable-p (cdr form))))))))
   #-sb-xc-host
   (flet ((expand (form)
            (if expand
@@ -121,11 +152,7 @@
                              (every #'fopcompilable-p args)))
                        ;; Allow SETQ only on special or global variables
                        ((setq)
-                        (loop for (name value) on args by #'cddr
-                              always (and (symbolp name)
-                                          (member (info :variable :kind name)
-                                                  '(:special :global))
-                                          (fopcompilable-p value))))
+                        (setq-fopcompilable-p args))
                        ;; The real toplevel form processing has already been
                        ;; done, so EVAL-WHEN handling will be easy.
                        ((eval-when)
@@ -166,7 +193,7 @@
                              ;; calling LIST, which is probably more
                              ;; trouble than it's worth).
                              (<= (length args) 255)
-                             (every #'fopcompilable-p args)))))))))))
+                             (every #'fopcompilable-p args))))))))))))
 
 (defun let-fopcompilable-p (operator args)
   (when (>= (length args) 1)
@@ -215,7 +242,7 @@
                ;; can't contain other objects
                ;; FIXME: OAOOM. See MAYBE-EMIT-MAKE-LOAD-FORMS.
                (unless (typep value
-                              '(or unboxed-array
+                              '(or #-sb-xc-host unboxed-array
                                 symbol
                                 number
                                 character
@@ -243,6 +270,11 @@
                    ((array t)
                     (dotimes (i (array-total-size value))
                       (grovel (row-major-aref value i))))
+                   ;; This is the same kludge as appears in EMIT-MAKE-LOAD-FORM
+                   ;; which informs the xc that LAYOUTs are leaf-like nodes.
+                   ;; This case was never reached before because cross-compiling
+                   ;; used to generate target machine code for everything.
+                   #+sb-xc-host (layout)
                    (instance
                     (multiple-value-bind (creation-form init-form)
                         (handler-case
@@ -346,6 +378,8 @@
                       ;; to record locations, which is strange because the main
                       ;; compiler does not have similar logic afaict.
                       ((source-location)
+                       ;; FIXME: since the fopcompiler expands compiler-macros,
+                       ;; this case should probably be killed. It can't execute.
                        (if (policy *policy* (and (> space 1)
                                                  (> space debug)))
                            (fopcompile-constant nil for-value-p)
@@ -356,17 +390,21 @@
                       ((if)
                        (fopcompile-if args path for-value-p))
                       ((progn locally)
-                       (loop for (arg . next) on args
+                       (if (and for-value-p (endp args))
+                           (fopcompile nil path t)
+                           (loop for (arg . next) on args
                              do (fopcompile arg
                                             path (if next
                                                      nil
-                                                     for-value-p))))
+                                                     for-value-p)))))
                       ((setq)
-                       (loop for (name value . next) on args by #'cddr
+                       (if (and for-value-p (endp args))
+                           (fopcompile nil path t)
+                           (loop for (name value . next) on args by #'cddr
                              do (fopcompile `(set ',name ,value) path
                                             (if next
                                                 nil
-                                                for-value-p))))
+                                                for-value-p)))))
                       ((eval-when)
                        (destructuring-bind (situations &body body) args
                          (if (or (member :execute situations)
