@@ -66,6 +66,7 @@
   "the fixnum closest in value to negative infinity")
 
 (def!constant most-positive-word (1- (expt 2 n-word-bits))
+  #!+sb-doc
   "The most positive integer that is of type SB-EXT:WORD.")
 
 (def!constant most-positive-exactly-single-float-fixnum
@@ -87,3 +88,8 @@
         (when (< start other-start)
           (setf stop (min stop other-start))))
       stop))
+
+;; The lowest index that you can pass to %INSTANCE-REF accessing
+;; a slot of data that is not the instance-layout.
+;; To get a layout, you must call %INSTANCE-LAYOUT - don't assume index 0.
+(def!constant instance-data-start 1)
