@@ -270,7 +270,7 @@
       (let ((kind (info :variable :kind name))
             (type (info :variable :type name))
             (where-from (info :variable :where-from name)))
-        (when (eq kind :unknown)
+        (when (and (eq kind :unknown) (not (check-deprecated-variable name)))
           (note-undefined-reference name :variable))
         (setf (gethash name *free-vars*)
               (case kind
@@ -689,24 +689,30 @@
                          `(symbol-value ',name)))
         (etypecase var
           (leaf
-           (when (lambda-var-p var)
-             (let ((home (ctran-home-lambda-or-null start)))
-               (when home
-                 (sset-adjoin var (lambda-calls-or-closes home))))
-             (when (lambda-var-ignorep var)
-               ;; (ANSI's specification for the IGNORE declaration requires
-               ;; that this be a STYLE-WARNING, not a full WARNING.)
-               #-sb-xc-host
-               (compiler-style-warn "reading an ignored variable: ~S" name)
-               ;; there's no need for us to accept ANSI's lameness when
-               ;; processing our own code, though.
-               #+sb-xc-host
-               (warn "reading an ignored variable: ~S" name)))
-           (when (global-var-p var)
-             (check-deprecated-variable name))
+           (cond
+             ((lambda-var-p var)
+              (let ((home (ctran-home-lambda-or-null start)))
+                (when home
+                  (sset-adjoin var (lambda-calls-or-closes home))))
+              (when (lambda-var-ignorep var)
+                ;; (ANSI's specification for the IGNORE declaration requires
+                ;; that this be a STYLE-WARNING, not a full WARNING.)
+                #-sb-xc-host
+                (compiler-style-warn "reading an ignored variable: ~S" name)
+                ;; there's no need for us to accept ANSI's lameness when
+                ;; processing our own code, though.
+                #+sb-xc-host
+                (warn "reading an ignored variable: ~S" name)))
+             (t
+              (multiple-value-bind (state since replacements)
+                  (check-deprecated-variable name)
+                (when (eq state :final)
+                  (ir1-convert
+                   start next result
+                   `(deprecation-error ,since ',name '(,@replacements)))
+                  (return-from ir1-convert-var (values))))))
            (reference-leaf start next result var name))
-          (cons
-           (aver (eq (car var) 'macro))
+          ((cons (eql macro)) ; symbol-macro
            ;; FIXME: [Free] type declarations. -- APD, 2002-01-26
            (ir1-convert start next result (cdr var)))
           (heap-alien-info
@@ -760,13 +766,9 @@
 (defun ir1-convert-functoid (start next result form)
   (let* ((op (car form))
          (translator (and (symbolp op) (info :function :ir1-convert op))))
-    (cond (translator
-           ;; FIXME: redundant? A macro can not be defined in the first place.
-           (when (sb!xc:compiler-macro-function op *lexenv*)
-             (compiler-warn "ignoring compiler macro for special form"))
-           (funcall translator start next result form))
-          (t
-           (multiple-value-bind (res cmacro-fun-name)
+    (if translator
+        (funcall translator start next result form)
+        (multiple-value-bind (res cmacro-fun-name)
                (expand-compiler-macro form)
              (cond ((eq res form)
                     (ir1-convert-common-functoid start next result form op))
@@ -774,7 +776,7 @@
                     (unless (policy *lexenv* (zerop store-xref-data))
                       (record-call cmacro-fun-name (ctran-block start)
                                    *current-path*))
-                    (ir1-convert start next result res))))))))
+                    (ir1-convert start next result res)))))))
 
 ;;; Handles the "common" cases: any other forms except special forms
 ;;; and compiler-macros.
@@ -1091,10 +1093,16 @@
           (if transform
               (multiple-value-bind (transformed pass)
                   (if (functionp transform)
-                      (funcall transform form)
-                      (let ((result (slot-access-transform
-                                     (if (consp name) :write :read)
-                                     (cdr form) transform)))
+                      (funcall transform form *lexenv*)
+                      (let ((result
+                             (if (eq (cdr transform) :predicate)
+                                 (and (singleton-p (cdr form))
+                                      `(%instance-typep
+                                        ,(cadr form)
+                                        ',(dd-name (car transform))))
+                                 (slot-access-transform
+                                  (if (consp name) :write :read)
+                                  (cdr form) transform))))
                         (values result (null result))))
                 (cond (pass
                        (ir1-convert-maybe-predicate start next result form var))
@@ -1240,7 +1248,7 @@
                            (lambda-var-specvar var))
                       (process-var it nil))))
             (cons
-             ;; FIXME: non-ANSI weirdness
+             ;; FIXME: non-ANSI weirdness. [See lp#309122]
              (aver (eq (car var) 'macro))
              (new-vars `(,var-name . (macro . (the ,(first decl)
                                                 ,(cdr var))))))
@@ -1619,8 +1627,7 @@
                 (process-it))
             ;; Kludge: EVAL calls this function to deal with LOCALLY.
               (process-it)))))
-    (advise-if-repeated-optimize-qualities
-     (lexenv-policy lexenv) optimize-qualities)
+    (warn-repeated-optimize-qualities (lexenv-policy lexenv) optimize-qualities)
     (values lexenv result-type *post-binding-variable-lexenv*)))
 
 (defun %processing-decls (decls vars fvars ctran lvar binding-form-p fun)
