@@ -820,14 +820,18 @@ necessary, since type inference may take arbitrarily long to converge.")
   ;; the file positions that reading of each form started at (i.e. the
   ;; end of the previous form)
   (forms (make-array 10 :fill-pointer 0 :adjustable t) :type (vector t))
-  (positions (make-array 10 :fill-pointer 0 :adjustable t) :type (vector t)))
+  (positions (make-array 10 :fill-pointer 0 :adjustable t) :type (vector t))
+  ;; A vector of character ranges than span each subform in the TLF,
+  ;; reset to empty for each one, updated by form-tracking-stream-observer.
+  (subforms nil :type (or null (vector t)) :read-only t))
 
 ;;; The SOURCE-INFO structure provides a handle on all the source
 ;;; information for an entire compilation.
 (def!struct (source-info
              #-no-ansi-print-object
              (:print-object (lambda (s stream)
-                              (print-unreadable-object (s stream :type t))))
+                              (print-unreadable-object
+                                  (s stream :type t :identity t))))
              (:copier nil))
   ;; the UT that compilation started at
   (start-time (get-universal-time) :type unsigned-byte)
@@ -847,11 +851,14 @@ necessary, since type inference may take arbitrarily long to converge.")
   (parent nil :type (or source-info null)))
 
 ;;; Given a pathname, return a SOURCE-INFO structure.
-(defun make-file-source-info (file external-format)
+(defun make-file-source-info (file external-format &optional form-tracking-p)
   (make-source-info
    :file-info (make-file-info :name (truename file)
                               :untruename (merge-pathnames file)
                               :external-format external-format
+                              :subforms
+                              (if form-tracking-p
+                                  (make-array 100 :fill-pointer 0 :adjustable t))
                               :write-date (file-write-date file))))
 
 ;;; Return a SOURCE-INFO to describe the incremental compilation of FORM.
@@ -874,39 +881,6 @@ necessary, since type inference may take arbitrarily long to converge.")
                 (pathnamep (file-info-name finfo)))
             finfo))))
 
-;;; Return a form read from STREAM; or for EOF use the trick,
-;;; popularized by Kent Pitman, of returning STREAM itself. If an
-;;; error happens, then convert it to standard abort-the-compilation
-;;; error condition (possibly recording some extra location
-;;; information).  CONDITION-NAME is what to signal on error,
-;;; and should be INPUT-ERROR-IN-COMPILE-FILE or a subclass of it.
-;;; The signaled condition encapsulates a reader condition.
-(defun read-for-compile-file (stream position condition-name)
-  (handler-case
-      (read-preserving-whitespace stream nil stream)
-    (reader-error (condition)
-      (compiler-error condition-name
-                      ;; We don't need to supply :POSITION here because
-                      ;; READER-ERRORs already know their position in the file.
-                      :condition condition
-                      :stream stream))
-    ;; ANSI, in its wisdom, says that READ should return END-OF-FILE
-    ;; (and that this is not a READER-ERROR) when it encounters end of
-    ;; file in the middle of something it's trying to read.
-    (end-of-file (condition)
-      (compiler-error condition-name
-                      :condition condition
-                      ;; We need to supply :POSITION here because the END-OF-FILE
-                      ;; condition doesn't carry the position that the user
-                      ;; probably cares about, where the failed READ began.
-                      :position position
-                      :stream stream))
-    (error (condition)
-      (compiler-error condition-name
-                      :condition condition
-                      :position position
-                      :stream stream))))
-
 ;;; If STREAM is present, return it, otherwise open a stream to the
 ;;; current file. There must be a current file.
 ;;;
@@ -927,8 +901,17 @@ necessary, since type inference may take arbitrarily long to converge.")
         (setf sb!xc:*compile-file-truename* name
               sb!xc:*compile-file-pathname* (file-info-untruename file-info)
               (source-info-stream info)
-              (open name :direction :input
-                    :external-format external-format)))))
+              (let ((stream
+                     (open name
+                           :direction :input
+                           :external-format external-format
+                           ;; SBCL stream classes aren't available in the host
+                           #-sb-xc-host :class
+                           #-sb-xc-host 'form-tracking-stream)))
+                (when (file-info-subforms file-info)
+                  (setf (form-tracking-stream-observer stream)
+                        (make-form-tracking-stream-observer file-info)))
+                stream)))))
 
 ;;; Close the stream in INFO if it is open.
 (defun close-source-info (info)
@@ -938,6 +921,65 @@ necessary, since type inference may take arbitrarily long to converge.")
   (setf (source-info-stream info) nil)
   (values))
 
+;; Loop over forms read from INFO's stream, calling FUNCTION with each.
+;; CONDITION-NAME is signaled if there is a reader error, and should be
+;; a subtype of not-so-aptly-named INPUT-ERROR-IN-COMPILE-FILE.
+(defun %do-forms-from-info (function info condition-name)
+  (declare (function function))
+  (let* ((file-info (source-info-file-info info))
+         (stream (get-source-stream info))
+         (pos (file-position stream))
+         (form
+          ;; Return a form read from STREAM; or for EOF use the trick,
+          ;; popularized by Kent Pitman, of returning STREAM itself.
+          (handler-case
+              (progn
+                ;; Reset for a new toplevel form.
+                (when (form-tracking-stream-p stream)
+                  (setf (form-tracking-stream-form-start-char-pos stream) nil))
+                (awhen (file-info-subforms file-info)
+                  (setf (fill-pointer it) 0))
+                (read-preserving-whitespace stream nil stream))
+            (reader-error (condition)
+              (compiler-error condition-name
+                ;; We don't need to supply :POSITION here because
+                ;; READER-ERRORs already know their position in the file.
+                              :condition condition
+                              :stream stream))
+            ;; ANSI, in its wisdom, says that READ should return END-OF-FILE
+            ;; (and that this is not a READER-ERROR) when it encounters end of
+            ;; file in the middle of something it's trying to read,
+            ;; making it unfortunately indistinguishable from legal EOF.
+            ;; Were it not for that, it would be more elegant to just
+            ;; handle one more condition in the HANDLER-CASE.
+            ((or end-of-file error) (condition)
+              (compiler-error
+               condition-name
+               :condition condition
+                ;; We need to supply :POSITION here because the END-OF-FILE
+                ;; condition doesn't carry the position that the user
+                ;; probably cares about, where the failed READ began.
+               :position
+               (or (and (form-tracking-stream-p stream)
+                        (form-tracking-stream-form-start-byte-pos stream))
+                   pos)
+               :line/col
+               (and (form-tracking-stream-p stream)
+                    (line/col-from-charpos
+                     stream
+                     (form-tracking-stream-form-start-char-pos stream)))
+               :stream stream)))))
+    (unless (eq form stream) ; not EOF
+      (funcall function form
+               :current-index
+               (let* ((forms (file-info-forms file-info))
+                      (current-idx (+ (fill-pointer forms)
+                                      (file-info-source-root file-info))))
+                 (vector-push-extend form forms)
+                 (vector-push-extend pos (file-info-positions file-info))
+                 current-idx))
+      (%do-forms-from-info function info condition-name))))
+
 ;;; Loop over FORMS retrieved from INFO.  Used by COMPILE-FILE and
 ;;; LOAD when loading from a FILE-STREAM associated with a source
 ;;; file.  ON-ERROR is the name of a condition class that should
@@ -948,22 +990,9 @@ necessary, since type inference may take arbitrarily long to converge.")
   (aver (symbolp form))
   (once-only ((info info))
     `(let ((*source-info* ,info))
-       (loop (destructuring-bind (,form &key ,@keys &allow-other-keys)
-                 (let* ((file-info (source-info-file-info ,info))
-                        (stream (get-source-stream ,info))
-                        (pos (file-position stream))
-                        (form (read-for-compile-file stream pos ,on-error)))
-                   (if (eq form stream) ; i.e., if EOF
-                       (return)
-                       (let* ((forms (file-info-forms file-info))
-                              (current-idx (+ (fill-pointer forms)
-                                              (file-info-source-root
-                                               file-info))))
-                         (vector-push-extend form forms)
-                         (vector-push-extend pos (file-info-positions
-                                                  file-info))
-                         (list form :current-index current-idx))))
-               ,@body)))))
+       (%do-forms-from-info (lambda (,form &key ,@keys &allow-other-keys)
+                              ,@body)
+                            ,info ,on-error))))
 
 ;;; Read and compile the source file.
 (defun sub-sub-compile-file (info)
@@ -1301,9 +1330,11 @@ necessary, since type inference may take arbitrarily long to converge.")
     (if (null *macro-policy*)
         (frob)
         (let* ((*lexenv*
-                (make-lexenv :policy (process-optimize-decl (macro-policy-decls nil)
-                                                            (lexenv-policy *lexenv*))
-                             :default *lexenv*))
+                (make-lexenv
+                 :policy (process-optimize-decl
+                          `(optimize ,@(policy-to-decl-spec *macro-policy*))
+                          (lexenv-policy *lexenv*))
+                 :default *lexenv*))
                ;; In case a null lexenv is created, it needs to get the newly
                ;; effective global policy, not the policy currently in *POLICY*.
                (*policy* (lexenv-policy *lexenv*)))
@@ -1842,7 +1873,9 @@ SPEED and COMPILATION-SPEED optimization values, and the
          (warnings-p nil)
          (failure-p t) ; T in case error keeps this from being set later
          (input-pathname (verify-source-file input-file))
-         (source-info (make-file-source-info input-pathname external-format))
+         (source-info
+          (make-file-source-info input-pathname external-format
+                                 #-sb-xc-host t)) ; can't track, no SBCL streams
          (*compiler-trace-output* nil)) ; might be modified below
 
     (unwind-protect
