@@ -670,10 +670,6 @@
   ;; operand SC restriction.
   (arg-load-scs nil :type list)
   (result-load-scs nil :type list)
-  ;; if true, a function that is called with the VOP to do operand
-  ;; targeting. This is done by modifying the TN-REF-TARGET slots in
-  ;; the TN-REFS so that they point to other TN-REFS in the same VOP.
-  (target-fun nil :type (or null function))
   ;; a function that emits assembly code for a use of this VOP when it
   ;; is called with the VOP structure. This is null if this VOP has no
   ;; specified generator (i.e. if it exists only to be inherited by
