@@ -41,10 +41,18 @@
     (reference-condition simple-error)
   ())
 
+(defun change-class-to-metaobject-violation (to-name
+                                             &optional from-name references)
+  (error 'metaobject-initialization-violation
+         :format-control "~@<Cannot ~S~@[ ~S~] objects into ~S metaobjects.~@:>"
+         :format-arguments (list 'change-class from-name to-name)
+         :references references))
+
 (macrolet ((def (name args control)
                `(defmethod ,name ,args
                  (declare (ignore initargs))
                  (error 'metaobject-initialization-violation
+                  ;; FIXME: I'm pretty sure this wants to be "~~@<~A~~@:>"
                   :format-control ,(format nil "~@<~A~@:>" control)
                   :format-arguments (list ',name)
                   :references (list '(:amop :initialization method))))))
@@ -467,14 +475,14 @@
 
 (defun real-add-method (generic-function method &optional skip-dfun-update-p)
   (flet ((similar-lambda-lists-p (old-method new-lambda-list)
-           (multiple-value-bind (a-nreq a-nopt a-keyp a-restp)
-               (analyze-lambda-list (method-lambda-list old-method))
-             (multiple-value-bind (b-nreq b-nopt b-keyp b-restp)
-                 (analyze-lambda-list new-lambda-list)
+           (binding* (((a-llks a-nreq a-nopt)
+                       (analyze-lambda-list (method-lambda-list old-method)))
+                      ((b-llks b-nreq b-nopt)
+                       (analyze-lambda-list new-lambda-list)))
                (and (= a-nreq b-nreq)
                     (= a-nopt b-nopt)
-                    (eq (or a-keyp a-restp)
-                        (or b-keyp b-restp)))))))
+                    (eq (ll-keyp-or-restp a-llks)
+                        (ll-keyp-or-restp b-llks))))))
     (multiple-value-bind (lock qualifiers specializers new-lambda-list
                           method-gf name)
         (values-for-add-method generic-function method)
@@ -611,26 +619,26 @@
                (gf-lambda-list (generic-function-lambda-list gf))
                (tfun (constantly t))
                keysp)
-          (multiple-value-bind (gf.required gf.optional gf.restp gf.rest
-                                            gf.keyp gf.keys gf.allowp)
+          (multiple-value-bind (llks gf.required gf.optional gf.rest gf.keys)
               (parse-lambda-list gf-lambda-list)
-            (declare (ignore gf.rest))
             ;; 7.6.4 point 5 probably entails that if any method says
             ;; &allow-other-keys then the gf should be construed to
             ;; accept any key.
-            (let* ((allowp (or gf.allowp
+            (let* ((allowp (or (ll-kwds-allowp llks)
                                (find '&allow-other-keys methods
                                      :test #'find
                                      :key #'method-lambda-list)))
                    (ftype
                     (specifier-type
+                     ;; SERIOUSLY? Do we not already have like at least
+                     ;; two other variations on this code?
                      `(function
                        (,@(mapcar tfun gf.required)
                           ,@(if gf.optional
                                 `(&optional ,@(mapcar tfun gf.optional)))
-                          ,@(if gf.restp
+                          ,@(if gf.rest
                                 `(&rest t))
-                          ,@(when gf.keyp
+                          ,@(when (ll-kwds-keyp llks)
                               (let ((all-keys
                                      (mapcar
                                       (lambda (x)
@@ -638,7 +646,7 @@
                                       (remove-duplicates
                                        (nconc
                                         (mapcan #'function-keywords methods)
-                                        (mapcar #'keyword-spec-name gf.keys))))))
+                                        (mapcar #'parse-key-arg-spec gf.keys))))))
                                 (when all-keys
                                   (setq keysp t)
                                   `(&key ,@all-keys))))
@@ -669,7 +677,7 @@
    generic-function
    (types-from-args generic-function classes 'class-eq)))
 
-(defun proclaim-incompatible-superclasses (classes)
+(defun !proclaim-incompatible-superclasses (classes)
   (setq classes (mapcar (lambda (class)
                           (if (symbolp class)
                               (find-class class)
@@ -689,9 +697,9 @@
           (return-from superclasses-compatible-p nil))))))
 
 (mapc
- #'proclaim-incompatible-superclasses
+ #'!proclaim-incompatible-superclasses
  '(;; superclass class
-   (built-in-class std-class structure-class) ; direct subclasses of pcl-class
+   (system-class std-class structure-class) ; direct subclasses of pcl-class
    (standard-class funcallable-standard-class)
    ;; superclass metaobject
    (class eql-specializer class-eq-specializer method method-combination
@@ -744,7 +752,6 @@
     (declare (ignore applyp metatypes nkeys))
     (let ((types-rev nil))
       (dotimes-fixnum (i nreq)
-        i
         (unless arguments
           (error-need-at-least-n-args (generic-function-name generic-function)
                                       nreq))
@@ -1085,7 +1092,7 @@
   (cond
     ((eq class *the-class-t*) t)
     ((eq class *the-class-slot-object*)
-     `(not (typep (classoid-of ,arg) 'built-in-classoid)))
+     `(not (typep (classoid-of ,arg) 'system-classoid)))
     ((eq class *the-class-standard-object*)
      `(or (std-instance-p ,arg) (fsc-instance-p ,arg)))
     ((eq class *the-class-funcallable-standard-object*)
@@ -1417,8 +1424,10 @@
                         (make-dfun-lambda-list nargs applyp)
                         (make-fast-method-call-lambda-list nargs applyp))))
       (multiple-value-bind (cfunction constants)
-          (get-fun1 `(named-lambda (gf-dispatch ,name)
-                      ,arglist
+          ;; We don't want NAMED-LAMBDA for any expressions handed to FNGEN,
+          ;; because name mismatches will render the hashing ineffective.
+          (get-fun1 `(lambda ,arglist
+                      (declare (optimize (sb-c::store-closure-debug-pointer 3)))
                       ,@(unless function-p
                           `((declare (ignore .pv. .next-method-call.))))
                       (locally (declare #.*optimize-speed*)
@@ -1445,7 +1454,7 @@
                 (setf alist-tail new)))
             (let ((function (apply cfunction (mapcar #'cdr (cdr alist)))))
               (if function-p
-                  function
+                  (set-fun-name function `(gf-dispatch ,name))
                   (make-fast-method-call
                    :function (set-fun-name function `(sdfun-method ,name))
                    :arg-info fmc-arg-info))))))))))
@@ -1673,28 +1682,26 @@
   new-value)
 
 (defmethod function-keywords ((method standard-method))
-  (multiple-value-bind (nreq nopt keysp restp allow-other-keys-p
-                        keywords)
+  (multiple-value-bind (llks nreq nopt keywords)
       (analyze-lambda-list (if (consp method)
                                (early-method-lambda-list method)
                                (method-lambda-list method)))
-    (declare (ignore nreq nopt keysp restp))
-    (values keywords allow-other-keys-p)))
+    (declare (ignore nreq nopt))
+    (values keywords (ll-kwds-allowp llks))))
 
 (defmethod function-keyword-parameters ((method standard-method))
-  (multiple-value-bind (nreq nopt keysp restp allow-other-keys-p
-                        keywords keyword-parameters)
+  (multiple-value-bind (llks nreq nopt keywords keyword-parameters)
       (analyze-lambda-list (if (consp method)
                                (early-method-lambda-list method)
                                (method-lambda-list method)))
-    (declare (ignore nreq nopt keysp restp keywords))
-    (values keyword-parameters allow-other-keys-p)))
+    (declare (ignore nreq nopt keywords))
+    (values keyword-parameters (ll-kwds-allowp llks))))
 
+;; FIXME: this is just: "parse, set keys to nil, unparse" (I think).
 (defun method-ll->generic-function-ll (ll)
-  (multiple-value-bind
-      (nreq nopt keysp restp allow-other-keys-p keywords keyword-parameters)
+  (multiple-value-bind (llks nreq nopt keywords keyword-parameters)
       (analyze-lambda-list ll)
-    (declare (ignore nreq nopt keysp restp allow-other-keys-p keywords))
+    (declare (ignore llks nreq nopt keywords))
     (remove-if (lambda (s)
                  (or (memq s keyword-parameters)
                      (eq s '&allow-other-keys)))
@@ -1715,8 +1722,8 @@
         (methods (generic-function-methods generic-function)))
     (if (null methods)
         gf-lambda-list
-        (multiple-value-bind (gf.required gf.optional gf.rest gf.keys gf.allowp)
-            (%split-arglist gf-lambda-list)
+        (multiple-value-bind (gf.llks gf.required gf.optional gf.rest gf.keys)
+            (parse-lambda-list gf-lambda-list :silent t)
           ;; Possibly extend the keyword parameters of the gf by
           ;; additional key parameters of its methods:
           (let ((methods.keys nil) (methods.allowp nil))
@@ -1726,7 +1733,7 @@
                 (setq methods.keys (union methods.keys m.keyparams :key #'maybe-car))
                 (setq methods.allowp (or methods.allowp m.allow-other-keys))))
             (let ((arglist '()))
-              (when (or gf.allowp methods.allowp)
+              (when (or (ll-kwds-allowp gf.llks) methods.allowp)
                 (push '&allow-other-keys arglist))
               (when (or gf.keys methods.keys)
                 ;; We make sure that the keys of the gf appear before
@@ -1736,7 +1743,7 @@
                                      (nset-difference methods.keys gf.keys)
                                      arglist)))
               (when gf.rest
-                (setq arglist (nconc (list '&rest gf.rest) arglist)))
+                (setq arglist (nconc (cons '&rest gf.rest) arglist)))
               (when gf.optional
                 (setq arglist (nconc (list '&optional) gf.optional arglist)))
               (nconc gf.required arglist)))))))
@@ -1745,14 +1752,3 @@
   (if (listp thing)
       (car thing)
       thing))
-
-
-(defun %split-arglist (lambda-list)
-  ;; This function serves to shrink the number of returned values of
-  ;; PARSE-LAMBDA-LIST to something handier.
-  (multiple-value-bind (required optional restp rest keyp keys allowp
-                        auxp aux morep more-context more-count)
-      (parse-lambda-list lambda-list :silent t)
-    (declare (ignore restp keyp auxp aux morep))
-    (declare (ignore more-context more-count))
-    (values required optional rest keys allowp)))
