@@ -14,7 +14,10 @@
 
 (cl:defpackage "ASSERTOID"
   (:use "CL")
-  (:export "GRAB-CONDITION" "RAISES-ERROR?" "IS" "ASSERTOID"))
+  (:export "GRAB-CONDITION" "ASSERT-ERROR"
+           "HAS-ERROR?" "IS" "ASSERTOID"
+           "ASSERT-SIGNAL" "ASSERT-NO-SIGNAL"
+           "EQUAL-MOD-GENSYMS"))
 
 (cl:in-package "ASSERTOID")
 
@@ -22,9 +25,28 @@
   `(nth-value 1
      (ignore-errors ,@body)))
 
-(defmacro raises-error? (form &optional (error-subtype-spec 'error))
+(defmacro has-error? (form &optional (error-subtype-spec 'error))
   `(typep (nth-value 1 (ignore-errors ,form)) ',error-subtype-spec))
 
+(defmacro assert-error (form &optional (error-subtype-spec 'error))
+  `(assert (typep (nth-value 1 (ignore-errors ,form)) ',error-subtype-spec)))
+
+(defmacro assert-signal (form &optional (signal-type 'condition))
+  (let ((signal (gensym)))
+    `(let (,signal)
+       (handler-bind ((,signal-type (lambda (c)
+                                      (setf ,signal c))))
+         ,form)
+       (assert ,signal))))
+
+(defmacro assert-no-signal (form &optional (signal-type 'condition))
+  (let ((signal (gensym)))
+    `(let (,signal)
+       (handler-bind ((,signal-type (lambda (c)
+                                      (setf ,signal c))))
+         ,form)
+       (assert (not ,signal)))))
+
 ;;; EXPR is an expression to evaluate (both with EVAL and with
 ;;; COMPILE/FUNCALL). EXTRA-OPTIMIZATIONS is a list of lists of
 ;;; optimizations to pass to (DECLARE (OPTIMIZE ..)), to cause the
@@ -124,3 +146,26 @@
                     expected-value real-value ',form))))
       `(unless ,form
          (error "~S evaluated to NIL" ',form))))
+
+;; Return T if two sexprs are EQUAL, considering uninterned symbols
+;; in expression A as EQ to one in B provided that there exists a
+;; mapping that makes the forms EQUAL.
+;; This is helpful when testing complicated macroexpanders.
+;; Note that this is much simpler than unification,
+;; because symbols can only be replaced by other symbols.
+(defun equal-mod-gensyms (a b &optional (pred #'equal))
+  (let ((subst-table (make-hash-table :test 'eq)))
+    (labels ((recurse (a b)
+               (cond ((and (consp a) (consp b))
+                      (and (recurse (car a) (car b))
+                           (recurse (cdr a) (cdr b))))
+                     ((and (symbolp a) (symbolp b))
+                      (multiple-value-bind (replacement found)
+                          (gethash a subst-table a)
+                        (or (eq replacement b)
+                            (and (not found)
+                                 (not (symbol-package a))
+                                 (setf (gethash a subst-table) b)))))
+                     (t ; strings, numbers
+                      (funcall pred a b)))))
+      (recurse a b))))
