@@ -18,28 +18,28 @@
 ;;; sbcl-0.7.8.19
 (locally
     (declare (optimize (safety 3)))
-  (assert (raises-error? (make-two-way-stream (make-string-output-stream)
-                                              (make-string-output-stream))
-                         type-error))
-  (assert (raises-error? (make-two-way-stream (make-string-input-stream "foo")
-                                              (make-string-input-stream "bar"))
-                         type-error))
+  (assert-error (make-two-way-stream (make-string-output-stream)
+                                     (make-string-output-stream))
+                type-error)
+  (assert-error (make-two-way-stream (make-string-input-stream "foo")
+                                     (make-string-input-stream "bar"))
+                type-error)
   ;; the following two aren't actually guaranteed, because ANSI, as it
   ;; happens, doesn't say "should signal an error" for
   ;; MAKE-ECHO-STREAM. It's still good to have, but if future
   ;; maintenance work causes this test to fail because of these
   ;; MAKE-ECHO-STREAM clauses, consider simply removing these clauses
   ;; from the test. -- CSR, 2002-10-06
-  (assert (raises-error? (make-echo-stream (make-string-output-stream)
-                                           (make-string-output-stream))
-                         type-error))
-  (assert (raises-error? (make-echo-stream (make-string-input-stream "foo")
-                                           (make-string-input-stream "bar"))
-                         type-error))
-  (assert (raises-error? (make-concatenated-stream
-                          (make-string-output-stream)
-                          (make-string-input-stream "foo"))
-                         type-error)))
+  (assert-error (make-echo-stream (make-string-output-stream)
+                                  (make-string-output-stream))
+                type-error)
+  (assert-error (make-echo-stream (make-string-input-stream "foo")
+                                  (make-string-input-stream "bar"))
+                type-error)
+  (assert-error (make-concatenated-stream
+                 (make-string-output-stream)
+                 (make-string-input-stream "foo"))
+                type-error))
 
 ;;; bug 225: STRING-STREAM was not a class
 (eval `(defgeneric bug225 (s)
@@ -71,16 +71,17 @@
 (let* ((p "this-file-will-exist")
        (stream (open p :direction :output :if-exists :error)))
   (assert (null (with-open-file (s p :direction :output :if-exists nil) s)))
-  (assert (raises-error?
-           (with-open-file (s p :direction :output :if-exists :error))))
+  (assert-error
+   (with-open-file (s p :direction :output :if-exists :error)))
   (close stream)
   (delete-file p))
 
-(assert (raises-error? (read-byte (make-string-input-stream "abc"))
-                       type-error))
-(assert (raises-error? (with-open-file (s "/dev/zero")
-                         (read-byte s))
-                       type-error))
+(assert-error (read-byte (make-string-input-stream "abc"))
+              type-error)
+(assert-error (with-open-file (s "/dev/zero")
+                (read-byte s))
+              #-win32 type-error
+              #+win32 sb-int:simple-file-error)
 ;;; bidirectional streams getting confused about their position
 (let ((p "bidirectional-stream-test"))
   (with-open-file (s p :direction :output :if-exists :supersede)
@@ -131,27 +132,28 @@
 
 ;;; CLOSING a non-new streams should not delete them, and superseded
 ;;; files should be restored.
-(let ((test "test-file-for-close-should-not-delete"))
-  (macrolet ((test-mode (mode)
-               `(progn
-                 (catch :close-test-exit
-                   (with-open-file (f test :direction :output :if-exists ,mode)
-                     (write-line "test" f)
-                     (throw :close-test-exit t)))
-                 (assert (and (probe-file test) ,mode)))))
-    (unwind-protect
-         (progn
-           (with-open-file (f test :direction :output)
-             (write-line "test" f))
-           (test-mode :append)
-           (test-mode :overwrite)
-           ;; FIXME: We really should recover supersede files as well, according to
-           ;; CLOSE in CLHS, but at the moment we don't.
-           ;; (test-mode :supersede)
-           (test-mode :rename)
-           (test-mode :rename-and-delete))
-      (when (probe-file test)
-        (delete-file test)))))
+(with-test (:name :test-file-for-close-should-not-delete :fails-on :win32)
+  (let ((test "test-file-for-close-should-not-delete"))
+    (macrolet ((test-mode (mode)
+                          `(progn
+                             (catch :close-test-exit
+                               (with-open-file (f test :direction :output :if-exists ,mode)
+                                               (write-line "test" f)
+                                               (throw :close-test-exit t)))
+                             (assert (and (probe-file test) ,mode)))))
+      (unwind-protect
+          (progn
+            (with-open-file (f test :direction :output)
+                            (write-line "test" f))
+            (test-mode :append)
+            (test-mode :overwrite)
+            ;; FIXME: We really should recover supersede files as well, according to
+            ;; CLOSE in CLHS, but at the moment we don't.
+            ;; (test-mode :supersede)
+            (test-mode :rename)
+            (test-mode :rename-and-delete))
+        (when (probe-file test)
+          (delete-file test))))))
 
 ;;; test for read-write invariance of signed bytes, from Bruno Haible
 ;;; cmucl-imp 2004-09-06
@@ -421,16 +423,7 @@
 ;;; writing looong lines. takes way too long and way too much space
 ;;; to test on 64 bit platforms
 #-#.(cl:if (cl:= sb-vm:n-word-bits 64) '(and) '(or))
-(progn
-  (defun write-n-chars (n stream)
-    (format t "~&/writing ~D chars on a single line~%" n)
-    (finish-output t)
-    (loop repeat n
-       do (write-char #\x stream))
-    (terpri stream)
-    n)
-
-  (let ((test "long-lines-write-test.tmp"))
+(let ((test "long-lines-write-test.tmp"))
     (unwind-protect
          (with-open-file (f test
                             :direction :output
@@ -438,9 +431,20 @@
                             :element-type 'character
                             :if-does-not-exist :create
                             :if-exists :supersede)
-         (write-n-chars (+ most-positive-fixnum 7) f))
+           (let* ((n (truncate most-positive-fixnum 16))
+                  (m 18)
+                  (p (* n m))
+                  (buffer (make-string n)))
+             (dotimes (i m)
+               (write-char #\.)
+               (finish-output)
+               (write-sequence buffer f))
+             (assert (= p (sb-impl::fd-stream-output-column f)))
+             (write-char #\! f)
+             (assert (= (+ 1 p) (sb-impl::fd-stream-output-column f)))
+             (assert (typep p 'bignum))))
       (when (probe-file test)
-        (delete-file test)))))
+        (delete-file test))))
 
 ;;; read-sequence misreported the amount read and lost position
 (let ((string (make-array (* 3 sb-impl::+ansi-stream-in-buffer-length+)
@@ -486,5 +490,297 @@
   (with-open-stream (stream (make-synonym-stream '*stream*))
     (assert (input-stream-p stream))))
 
+;;; READ-LINE on ANSI-STREAM did not return T for the last line
+;;; (reported by Yoshinori Tahara)
+(let ((pathname "test-read-line-eol"))
+  (with-open-file (out pathname :direction :output :if-exists :supersede)
+    (format out "a~%b"))
+  (let ((result (with-open-file (in pathname)
+                  (list (multiple-value-list (read-line in nil nil))
+                        (multiple-value-list (read-line in nil nil))
+                        (multiple-value-list (read-line in nil nil))))))
+    (delete-file pathname)
+    (assert (equal result '(("a" nil) ("b" t) (nil t))))))
+
+;;; READ-LINE used to work on closed streams because input buffers were left in place
+(with-test (:name :bug-425)
+  ;; Normal close
+  (let ((f (open "stream.impure.lisp" :direction :input)))
+    (assert (stringp (read-line f)))
+    (close f)
+    (assert (eq :fii
+                (handler-case
+                    (read-line f)
+                  (sb-int:closed-stream-error () :fii)))))
+  ;; Abort
+  (let ((f (open "stream.impure.lisp" :direction :input)))
+    (assert (stringp (read-line f nil nil)))
+    (close f :abort t)
+    (assert (eq :faa
+                (handler-case
+                    (read-line f)
+                  (sb-int:closed-stream-error () :faa))))))
+
+(with-test (:name :regression-1.0.12.22)
+  (with-open-file (s "stream.impure.lisp" :direction :input)
+    (let ((buffer (make-string 20)))
+      (assert (= 2 (read-sequence buffer s :start 0 :end 2)))
+      (assert (= 3 (read-sequence buffer s :start 2 :end 3)))
+      (file-position s :end)
+      (assert (= 3 (read-sequence buffer s :start 3))))))
+
+;;; In 1.0.27 (and also 0.9.16; presumably in between, too), binary
+;;; input operations on a bivalent stream did something bad after
+;;; unread-char: READ-BYTE would return the character, and
+;;; READ-SEQUENCE into a byte buffer would lose when attempting to
+;;; store the character in the vector.
+(let ((pathname "bivalent-stream-unread-char-test.tmp"))
+  (with-open-file (s pathname
+                     :element-type :default
+                     :direction :io :if-exists :rename)
+    (write-char #\a s)
+    (file-position s :start)
+    (unread-char (read-char s) s)
+    (assert (integerp (read-byte s))))
+  (delete-file pathname))
+
+(let ((pathname "bivalent-stream-unread-char-test.tmp"))
+  (with-open-file (s pathname
+                     :element-type :default
+                     :direction :io :if-exists :rename)
+    (write-char #\a s)
+    (file-position s :start)
+    (unread-char (read-char s) s)
+    (assert (let ((buffer (make-array 10 :element-type '(unsigned-byte 8))))
+              (read-sequence buffer s))))
+  (delete-file pathname))
+
+#+sb-unicode
+(let ((pathname "bivalent-stream-unread-char-test.tmp"))
+  (with-open-file (s pathname
+                     :element-type :default
+                     :direction :io :if-exists :rename
+                     :external-format :utf8)
+    (write-char (code-char 192) s)
+    (file-position s :start)
+    (unread-char (read-char s) s)
+    (assert (integerp (read-byte s))))
+  (delete-file pathname))
+
+#+sb-unicode
+(let ((pathname "bivalent-stream-unread-char-test.tmp"))
+  (with-open-file (s pathname
+                     :element-type :default
+                     :direction :io :if-exists :rename
+                     :external-format :utf8)
+    (write-char (code-char 192) s)
+    (file-position s :start)
+    (unread-char (read-char s) s)
+    (assert (let ((buffer (make-array 10 :element-type '(unsigned-byte 8))))
+              (read-sequence buffer s))))
+  (delete-file pathname))
+
+(with-test (:name :delete-file-on-streams)
+  (with-open-file (f "delete-file-on-stream-test.tmp"
+                     :direction :io)
+    (delete-file f)
+    #-win32
+    (progn
+      (write-line "still open" f)
+      (file-position f :start)
+      (assert (equal "still open" (read-line f)))))
+  (assert (not (probe-file "delete-file-on-stream-test.tmp"))))
+
+;;; READ-CHAR-NO-HANG on bivalent streams (as returned by RUN-PROGRAM)
+;;; was wrong.  CSR managed to promote the wrongness to all streams in
+;;; the 1.0.32.x series, breaking slime instantly.
+(with-test (:name :read-char-no-hang-after-unread-char :skipped-on :win32)
+  (let* ((process (run-program "/bin/sh" '("-c" "echo a && sleep 10")
+                               :output :stream :wait nil))
+         (stream (process-output process))
+         (char (read-char stream)))
+    (assert (char= char #\a))
+    (unread-char char stream)
+    (assert (char= (read-char stream) #\a))
+    (assert (char= (read-char stream) #\Newline))
+    (let ((time (get-universal-time)))
+      ;; no input, not yet known to be at EOF: should return
+      ;; immediately
+      (read-char-no-hang stream)
+      (assert (< (- (get-universal-time) time) 2)))))
+
+(require :sb-posix)
+#-win32
+(with-test (:name :interrupt-open :skipped-on :win32)
+  (let ((fifo nil)
+        (to 0))
+    (unwind-protect
+         (progn
+           ;; Make a FIFO
+           (setf fifo (sb-posix:mktemp "SBCL-fifo.XXXXXXX"))
+           (sb-posix:mkfifo fifo (logior sb-posix:s-iwusr sb-posix:s-irusr))
+           ;; Try to open it (which hangs), and interrupt ourselves with a timer,
+           ;; continue (this used to result in an error due to open(2) returning with
+           ;; EINTR, then interupt again and unwind.
+           (handler-case
+               (with-timeout 2
+                 (handler-bind ((timeout (lambda (c)
+                                           (when (eql 1 (incf to))
+                                             (continue c)))))
+                   (with-timeout 1
+                     (with-open-file (f fifo :direction :input)
+                       :open))))
+             (timeout ()
+               (if (eql 2 to)
+                   :timeout
+                   :wtf))
+             (error (e)
+               e)))
+      (when fifo
+        (ignore-errors (delete-file fifo))))))
+
+#-win32
+(with-test (:name :overeager-character-buffering :skipped-on :win32)
+  (let ((fifo nil)
+        (proc nil))
+    (maphash
+     (lambda (format _)
+       (declare (ignore _))
+       (format t "trying ~A~%" format)
+       (finish-output t)
+       (unwind-protect
+            (progn
+              (setf fifo (sb-posix:mktemp "SBCL-fifo-XXXXXXX"))
+              (sb-posix:mkfifo fifo (logior sb-posix:s-iwusr sb-posix:s-irusr))
+              ;; KLUDGE: because we have both ends in the same process, we would
+              ;; need to use O_NONBLOCK, but this works too.
+              (setf proc
+                    (run-program "/bin/sh"
+                                 (list "-c"
+                                       (format nil "cat > ~A" (native-namestring fifo)))
+                                 :input :stream
+                                 :wait nil
+                                 :external-format format))
+              (write-line "foobar" (process-input proc))
+              (finish-output (process-input proc))
+              (with-open-file (f fifo :direction :input :external-format format)
+                (assert (equal "foobar" (read-line f)))))
+         (when proc
+           (ignore-errors
+             (close (process-input proc) :abort t)
+             (process-wait proc))
+           (ignore-errors (process-close proc))
+           (setf proc nil))
+         (when fifo
+           (ignore-errors (delete-file fifo))
+           (setf fifo nil))))
+     sb-impl::*external-formats*)))
+
+(with-test (:name :bug-657183 :skipped-on '(not :sb-unicode))
+  #+sb-unicode
+  (let ((name (merge-pathnames "stream-impure.temp-test"))
+        (text '(#\GREEK_SMALL_LETTER_LAMDA
+                #\JAPANESE_BANK_SYMBOL
+                #\Space
+                #\HEAVY_BLACK_HEART))
+        (positions '(2 5 6 9))
+        (sb-impl::*default-external-format* :utf-8))
+    (unwind-protect
+         (progn
+           (with-open-file (f name :external-format :default :direction :output
+                              :if-exists :supersede)
+             (assert (eql 0 (file-position f)))
+             (mapc (lambda (char pos)
+                     (write-char char f)
+                     (assert (eql pos (file-position f))))
+                   text
+                   positions))
+           (with-open-file (f name :external-format :default :direction :input)
+             (assert (eql 0 (file-position f)))
+             (assert (eql (pop text) (read-char f)))
+             (assert (eql (file-position f) 2))
+             (assert (eql (pop text) (read-char f)))
+             (assert (eql (file-position f) 5))
+             (assert (eql (pop text) (read-char f)))
+             (assert (eql (file-position f) 6))
+             (assert (eql (pop text) (read-char f)))
+             (assert (eql (file-position f) 9))
+             (assert (eql (file-length f) 9))))
+      (ignore-errors (delete-file name)))))
+
+(with-test (:name :bug-561642)
+  (let ((p "bug-561642-test.tmp"))
+    (unwind-protect
+         (progn
+           (with-open-file (f p
+                              :if-exists :supersede
+                              :if-does-not-exist :create
+                              :direction :output)
+             (write-line "FOOBAR" f))
+           (with-open-file (f p
+                              :if-exists :append
+                              :direction :output)
+             (let ((p0 (file-position f))
+                   (p1 (progn
+                         (write-char #\newline f)
+                         (file-position f)))
+                   (p2 (progn
+                         (write-char #\newline f)
+                         (finish-output f)
+                         (file-position f))))
+               (assert (eql 7 p0))
+               (assert (eql 8 p1))
+               (assert (eql 9 p2)))))
+      (ignore-errors (delete-file p)))))
+
+(defstruct (mock-fd-stream
+            (:constructor %make-mock-fd-stream (buffer-chain))
+            (:include sb-impl::ansi-stream
+                      (in #'mock-fd-stream-in)
+                      (n-bin #'mock-fd-stream-n-bin)
+                      (cin-buffer
+                       (make-array sb-impl::+ansi-stream-in-buffer-length+
+                                   :element-type 'character))))
+  buffer-chain)
+
+(defun make-mock-fd-stream (buffer-chain)
+  ;; For notational convenience, #\| becomes #\Newline.
+  (%make-mock-fd-stream
+   (mapcar (lambda (x) (substitute #\Newline #\| x)) buffer-chain)))
+
+(defun mock-fd-stream-in (stream eof-err-p eof-val)
+  (sb-impl::eof-or-lose stream eof-err-p eof-val))
+
+(defun mock-fd-stream-n-bin (stream char-buf start count eof-err-p)
+  (cond ((mock-fd-stream-buffer-chain stream)
+         (let* ((chars (pop (mock-fd-stream-buffer-chain stream)))
+                (n-chars (length chars)))
+           ;; make sure the mock object is being used as expected.
+           (assert (>= count (length chars)))
+           (replace char-buf chars :start1 start)
+           n-chars))
+        (t
+         (sb-impl::eof-or-lose stream eof-err-p 0))))
+
+(with-test (:name :read-chunk-from-frc-buffer)
+  (let ((s (make-mock-fd-stream '("zabc" "d" "efgh" "foo|bar" "hi"))))
+    (multiple-value-bind (line eofp)
+        (sb-impl::ansi-stream-read-line-from-frc-buffer s nil 'woot)
+      (assert (and (string= line "zabcdefghfoo") (not eofp))))
+    (multiple-value-bind (line eofp)
+        (sb-impl::ansi-stream-read-line-from-frc-buffer s nil 'woot)
+      (assert (and (string= line "barhi") eofp)))
+    (multiple-value-bind (line eofp)
+        (sb-impl::ansi-stream-read-line-from-frc-buffer s nil 'woot)
+      (assert (and (eq line 'woot) eofp))))
+  (let ((s (make-mock-fd-stream '("zabc" "d" "efgh" "foo*bar" "hi")))
+        (string (make-string 100)))
+    (let ((endpos
+           (sb-impl::ansi-stream-read-string-from-frc-buffer string s 10 nil)))
+      (assert (and (= endpos 28)
+                   (string= (subseq string 10 endpos) "zabcdefghfoo*barhi"))))
+    (let ((endpos
+           (sb-impl::ansi-stream-read-string-from-frc-buffer string s 0 nil)))
+      (assert (= endpos 0)))))
 
 ;;; success
